package nnga;
/*
 * Copyright (C) 2009 Istvan Fehervari, Wilfried Elmenreich
 * Original project page: http://www.frevotool.tk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License Version 3 as published
 * by the Free Software Foundation http://www.gnu.org/licenses/gpl-3.0.txt
 *
 * There is no warranty for this free software. The GPL requires that 
 * modified versions be marked as changed, so that their problems will
 * not be attributed erroneously to authors of previous versions.
 */
import interfaces.IMethod;
import interfaces.IRanking;
import interfaces.IRepresentation;
import interfaces.PropEntry;

import java.io.File;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import main.FrevoMain;

import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.Node;

import utils.SafeSAX;
import utils.StatKeeper;
import AbsoluteRanking.AbsoluteRanking;

public class NNGA extends IMethod {

	private int POPULATIONNUMBER; //number of parallel populations
	private int POPULATIONSIZE; //number of entities in each population
	private int GENERATIONS; //number of generations to calculate
	protected int P_ELITE; // % for elite selection
	protected int N_ELITE; // number of elite guys to keep
	protected int P_RANDOM;
	protected int N_RANDOM;
	protected int N_MUTATE;
	protected int P_MUTATE;
	protected int N_RENEW; // number of newly randomized guys
	protected int P_RENEW;
	protected int P_XOVER;
	protected int N_XOVER;
	protected int N_INTERXOVER_FREQ;
	protected int MUTATIONMETHOD,XOVERMETHOD;
	protected int SAVEINTERVAL;
	protected String SAVETO;//directory to save to
	private LinkedList<SimplePopulation> pops = new LinkedList<SimplePopulation>();
	protected int startgeneration = 0;
	protected int generation;
	
	private StatKeeper bfitness;
	private StatKeeper index;
	
	//IProblem requirements
	public int input,output;
	private File resultfile;
	
	private void initialize() {
		//Load NNGA parameters
		PropEntry popnum = getProperties().get("populationnumber");
		POPULATIONNUMBER = Integer.parseInt(popnum.getValue());
		PropEntry popsize = getProperties().get("populationsize");
		POPULATIONSIZE = Integer.parseInt(popsize.getValue());
		PropEntry gensize = getProperties().get("generations");
		GENERATIONS = Integer.parseInt(gensize.getValue());
		
		PropEntry p_elite = getProperties().get("percentage_elite");
		P_ELITE = Integer.parseInt(p_elite.getValue());
		PropEntry p_random = getProperties().get("percentage_random");
		P_RANDOM = Integer.parseInt(p_random.getValue());
		PropEntry p_mutate = getProperties().get("percentage_mutate");
		P_MUTATE = Integer.parseInt(p_mutate.getValue());		
		PropEntry p_renew = getProperties().get("percentage_renew");
		P_RENEW = Integer.parseInt(p_renew.getValue());
		PropEntry p_xover = getProperties().get("percentage_xover");
		P_XOVER = Integer.parseInt(p_xover.getValue());
		PropEntry n_inter = getProperties().get("interXover_frequency");
		N_INTERXOVER_FREQ = Integer.parseInt(n_inter.getValue());
		PropEntry mm = getProperties().get("mutationMethod");
		MUTATIONMETHOD = Integer.parseInt(mm.getValue());
		PropEntry xm = getProperties().get("xoverMethod");
		XOVERMETHOD = Integer.parseInt(xm.getValue());
		
		N_ELITE = POPULATIONSIZE*P_ELITE/100;
		N_RANDOM = POPULATIONSIZE*P_RANDOM/100;
		N_MUTATE = POPULATIONSIZE*P_MUTATE/100;
		N_RENEW = POPULATIONSIZE*P_RENEW/100;
		N_XOVER = POPULATIONSIZE*P_XOVER/100;
		
		int roundingError = POPULATIONSIZE-N_ELITE-N_RANDOM-N_MUTATE-N_RENEW-N_XOVER;
		
		N_ELITE += roundingError;
		
		PropEntry saveint = getProperties().get("saveinterval");
		SAVEINTERVAL = Integer.parseInt(saveint.getValue());
		
		PropEntry saveto = getProperties().get("saveto");
		SAVETO = saveto.getValue();
		
		//Get IProblem's requirements
		Hashtable <String, PropEntry> req = problem.getRequirements();
		PropEntry input = req.get("inputnumber");
		this.input = Integer.parseInt(input.getValue());
		
		PropEntry output = req.get("outputnumber");
		this.output = Integer.parseInt(output.getValue());
	}
	
	@Override
	public void run() {
		int lastsaved=-1;
		isRunning = true;
		initialize();
		
		progress = 0;
		
		if (resultfile == null) { //no file to load
			if ((FrevoMain.getResults() != null) && (FrevoMain.getResults().getClass() == ResultObject.class  )) { //there are results to be loaded
				ResultObject result = (ResultObject)FrevoMain.getResults();
				//System.out.println ("Intermediate results successfully loaded");				
				if (result.remainingGenerations != 0) { //there are still generations left
					pops =  result.populations;
					startgeneration = GENERATIONS - result.remainingGenerations;
					
					//FrevoMain.STATISTICS.clear();
					//index = result.statistics.get(0);
					//bfitness = result.statistics.get(1);
					index = FrevoMain.STATISTICS.get(0);
					bfitness = FrevoMain.STATISTICS.get(1);
					
				}
			} else { //Start from scratch, no populations to load
				for(int i = 0;i<POPULATIONNUMBER;i++)
				{
					pops.add(new SimplePopulation(POPULATIONSIZE,representation,this));
				}
				//FrevoMain.STATISTICS.clear();
				index = new StatKeeper(true,"Generations");
				bfitness = new StatKeeper(true,"Best Fitness."+FrevoMain.currentRun);
			}
						
		} else { //load populations from file - for continuing
			Document doc = SafeSAX.read(resultfile);
	        
	        Node dpopulations = doc.selectSingleNode("/frevo/populations");
			startgeneration = Integer.parseInt(dpopulations.valueOf( "./@generation" ));
			this.setRandomSeed(Long.parseLong(dpopulations.valueOf( "./@randomseed" )));
			List<?> npops = dpopulations.selectNodes(".//population");
			Iterator<?> it = npops.iterator();
			while(it.hasNext())
			{
				Node pop = (Node) it.next();
				
				ArrayList<IRepresentation> population = createList(pop);
				pops.add(new SimplePopulation(population,representation,this));
			}
		}
		
		//if the statistics object has no index column then generate it as well		
		if (FrevoMain.STATISTICS.size() == 0) {
			FrevoMain.STATISTICS.add(index);
		}
		
		//Collect best fitness		
		if (!FrevoMain.STATISTICS.contains(bfitness)) 
			FrevoMain.STATISTICS.add(bfitness);
		
		//Iterate through
		for (generation = startgeneration; generation < GENERATIONS; generation++) {
			System.out.println ("Proceeding to generation "+generation);
			
			setProgress(100*generation/GENERATIONS);
			
			if (FrevoMain.graphics) {
				if (isInterrupted) {
					//Publish results in FrevoMain
					ResultObject res = new ResultObject(pops,GENERATIONS-generation);
//					res.statistics.clear();
//					res.statistics.add(index);
//					res.statistics.add(bfitness);
					FrevoMain.publishResults(res);
					return;
				}
			}
			
			//Rank each population using ranking
			for (int k=0; k<POPULATIONNUMBER;k++) {
				IRanking r;
				try {
					r = (IRanking)(ranking.prepareComponent());
					if (FrevoMain.graphics) {
						if (isInterrupted){
							//Publish results in FrevoMain
							ResultObject res = new ResultObject(pops,GENERATIONS-generation);
//							res.statistics.clear();
//							res.statistics.add(index);
//							res.statistics.add(bfitness);
							FrevoMain.publishResults(res);
							return;
						}
					}
					r.rank(pops.get(k).getMembers(),problem,this);
					if (r instanceof AbsoluteRanking) {
						bfitness.add(AbsoluteRanking.BESTFITNESS);
					}
					//sort according to score
										
				} catch (InstantiationException e) {
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}
				if (FrevoMain.graphics) {
					if (isInterrupted) {
						//Publish results in FrevoMain
						ResultObject res = new ResultObject(pops,GENERATIONS-generation);
//						res.statistics.clear();
//						res.statistics.add(index);
//						res.statistics.add(bfitness);
						FrevoMain.publishResults(res);
						return;
					}
				}
			}
			
			//Should save every Nth population
			if (SAVEINTERVAL != 0) {
				if ((generation % SAVEINTERVAL) == 0) {
						DecimalFormat fm = new DecimalFormat("000");
						FrevoMain.saveResult(SAVETO+outputname+"_g"+fm.format(generation), this);
						lastsaved=generation;
				}
			}
			
			//Mutate all populations except when this is the last generation
			if (generation != GENERATIONS-1) {
				for (int j=0; j<POPULATIONNUMBER;j++) {
					mutateAll(pops.get(j));
				}
			}
			//co-evolve seedlist
	/*		if (cdrones.seedList != null) {
				//System.out.println ("Shuffling");
				Collections.sort(cdrones.seedList);
				//replace second half
				int size = cdrones.seedList.size();
				for (int k=0;k<size;k++) {
					cdrones.seedList.get(k).succeeded = 0;
					if(k>=(size/2))	cdrones.seedList.get(k).seed = generator.nextLong();				
				}
							
			} else {}//System.out.println ("Cannot shuffle seedlist!");
		*/	
			//Add statistics
			if (index != null) index.add(generation);
			
		}
		setProgress(100);
		
		//Publish results in FrevoMain
		ResultObject res = new ResultObject(pops,0);
//		res.statistics.clear();
//		res.statistics.add(index);
//		res.statistics.add(bfitness);
		FrevoMain.publishResults(res);
		
		//Save final solution
		System.out.println("Saving...");
		
		if (lastsaved < generation-1) {
			DecimalFormat fm = new DecimalFormat("000");
			FrevoMain.saveResult(SAVETO+outputname+"_g"+fm.format(generation-1), this);
		}
		//cdrones.seedList = null;
	}
	
	private class ResultObject {
		public LinkedList<SimplePopulation> populations;
		public int remainingGenerations;
		//public ArrayList<StatKeeper> statistics = new ArrayList<StatKeeper>();
		
		public ResultObject (LinkedList<SimplePopulation> pops, int remaininggenerations) {
			populations = pops;
			remainingGenerations = remaininggenerations;
		}
	}
	
	private ArrayList<IRepresentation> createList(Node nd) {
		ArrayList<IRepresentation> result = new ArrayList<IRepresentation>();
		
		try {
			List<?> npops = nd.selectNodes("./*");
			Iterator<?> it = npops.iterator();
			int size = npops.size();
			while(it.hasNext())
			{
				Node net = (Node) it.next();
				size--;
				if (size%10==0)
					size = size + (2*2-4);
				IRepresentation member = (IRepresentation)(representation.prepareComponent());
				member.loadFromXML (net,this.getRandom());
				result.add(member);
			}
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
				
		return result;
	}
	
//	private void saveGeneration(String name, int aktgeneration)throws RemoteException {
//		Document doc = DocumentHelper.createDocument();
//        doc.addDocType("frevo", null, System.getProperty("user.dir")+"//Components//ISave.dtd");
//        Element dfrevo = doc.addElement("frevo");
//        try {
//            //export problem
//        	Element problemsettings = dfrevo.addElement("problem");
//        	problemsettings.addAttribute("class", problem.getClassName());
//        	Vector<String> keys =  new Vector<String>(problem.getProperties().keySet());
//        	for(String k:keys)
//        	{
//        		Element entry = problemsettings.addElement("problementry");
//        		entry.addAttribute("key", k);
//        		entry.addAttribute("type",problem.getType(k).toString());
//        		entry.addAttribute("value", problem.getValue(k));
//        	}
//        	
//        	//export method (this basically)
//        	Element methodsettings = dfrevo.addElement("method");
//        	methodsettings.addAttribute("class", this.getClassName());
//        	keys =  new Vector<String>(this.getProperties().keySet());
//        	for(String k:keys)
//        	{
//        		Element entry = methodsettings.addElement("methodentry");
//        		entry.addAttribute("key", k);
//        		entry.addAttribute("type",this.getType(k).toString());
//        		entry.addAttribute("value", this.getValue(k));
//        	}
//        	
//        	//export ranking
//        	Element rankingsettings = dfrevo.addElement("ranking");
//        	rankingsettings.addAttribute("class", ranking.getClassName());
//        	keys =  new Vector<String>(ranking.getProperties().keySet());
//        	for(String k:keys)
//        	{
//        		Element entry = rankingsettings.addElement("rankingentry");
//        		entry.addAttribute("key", k);
//        		entry.addAttribute("type",ranking.getType(k).toString());
//        		entry.addAttribute("value", ranking.getValue(k));
//        	}
//        	
//            //export representation
//        	Element repsettings = dfrevo.addElement("representation");
//        	repsettings.addAttribute("class", representation.getClassName());
//        	keys =  new Vector<String>(representation.getProperties().keySet());
//        	for(String k:keys)
//        	{
//        		Element entry = repsettings.addElement("representationentry");
//        		entry.addAttribute("key", k);
//        		entry.addAttribute("type",representation.getType(k).toString());
//        		entry.addAttribute("value", representation.getValue(k));
//        	}
//        	
//            //export populations
//            Element dpopulations = dfrevo.addElement("populations");
//    		dpopulations.addAttribute("count",String.valueOf(POPULATIONNUMBER));
//    		dpopulations.addAttribute("generation",String.valueOf(aktgeneration));
//    		dpopulations.addAttribute("randomseed",String.valueOf(this.getSeed()));
//    		for(SimplePopulation pop:pops)
//    		{
//    			/*Element dpop = */pop.exportXml(dpopulations);
//    		}
//            
//    		//(new File("./Results/"+name)).mkdir();
//            File file = new File(SAVETO+name+".zre");
//         //   System.out.println("Writing XML-Tree to file " + file.getAbsolutePath());
//
//            OutputFormat format = OutputFormat.createPrettyPrint();
//            format.setLineSeparator(System.getProperty("line.separator"));
//
//            file.createNewFile();
//            FileWriter out = new FileWriter(file);
//            BufferedWriter bw = new BufferedWriter(out);
//            XMLWriter wr = new XMLWriter(bw, format);
//            wr.write(doc);
//            wr.close();
//            System.out.println("XML Writing Completed: "+name);
//        } catch (OutOfMemoryError mem) {
//            System.err.println("Could not export! (Out of memory)");
//        } catch (IOException e) {
//            System.err.println("IOException while writing to XML!");
//            e.printStackTrace();
//        }
//	}

	private void mutateAll(SimplePopulation simplePopulation) {
		//evolve all populations using the genetic algorithm
		for(SimplePopulation pop: pops)	{
			
			int position = 0; //track current position in the new array
			IRepresentation[] nets = pop.getNetArray(); //temporary array for new candidates
			
			// keep elite
			for(int i = position; i < N_ELITE; i++)
			{
				//Best Logging
			//	nets[i].getGAHistory().setEntry(generation,GAHistory.ELITE,nets[i].getRank(),nets[i].getScore());
			}
			position = N_ELITE;
			
			//select randomly, probability should be higher for better-ranked individuals
			for(int i = position;i<position+N_RANDOM;i++)
			{
				int randrange;
				int startind;
				
				// number of nets that are considered based on maximum diversity to 
				// individuals already considered so far
				// set this value to one to turn off this feature
				int diffsearch = 5;
				//TODO: calculation of startind bugged!
				//create a triangle distribution by adding two independent randoms
				randrange = POPULATIONSIZE-1 - (i+diffsearch-1);
				startind = rndIndex(randrange) + rndIndex(randrange-1) - (randrange-1); 
				
				if (startind < 0) startind= -startind;
				
				startind+=i;
				
				double bestdiff=-1;
				int srcind = startind;
				
				for(int j= startind; j<startind+diffsearch; j++) {
					double diff=1;
					//calculate product of diffs of all nets selected so far (0 to i-1) 
					for(int k=0; k<i; k++)
						diff *= nets[j].diffTo(nets[k]);
					if (diff > bestdiff) {
						bestdiff=diff;
						srcind=j;
					}
				}
				
				IRepresentation temp = nets[i];
				nets[i] = nets[srcind];
				nets[srcind] = temp;
//				//History log
			//	nets[i].getGAHistory().setEntry(generation,GAHistory.LUCKY,nets[i].getRank(),nets[i].getScore());
			}
			position += N_RANDOM;
			
			int survivors = position;
			
			//create mutated copies of selected nets
			for(int i = position;i<position+N_MUTATE;i++)
			{
				int srcind = rndIndex(survivors);
				nets[i] = nets[srcind].clone();
				nets[i].mutate(100,10,MUTATIONMETHOD);
//				//History log
		//		nets[i].getGAHistory().setEntry(generation,GAHistory.MUT,nets[i].getRank(),nets[i].getScore());
				
		//		nets[i].setScore(0);
//				//log.println("diff after mutation "+nets[i].diffTo(nets[srcind]));
			}
			position += N_MUTATE;
			
			//create some new individuals with random parameters 
			for(int i = position;i<position+N_RENEW;i++)
			{
				//GAHistory h = nets[i].getGAHistory();
				try {
					nets[i] = pop.genNewNetwork();
				} catch (InstantiationException e) {
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}

		//		nets[i].setGAHistory(h);
				//History log
			//	nets[i].getGAHistory().setEntry(generation,GAHistory.RENEW,i,0);
				
			//	nets[i].setScore(0);
			}
			position += N_RENEW;
			
			//create offsprings between two individuals
			for(int i = position;i<position+N_XOVER;i++) {
				IRepresentation mother, father;
				SimplePopulation fatherpop;
				int motherindex,fatherindex;//+ type
				
				motherindex = rndIndex(N_ELITE + N_RANDOM);
				mother = nets[motherindex];
				
				if(generation % N_INTERXOVER_FREQ == 0 && pops.size()>1)
				{
					// Xover between different Populations
					fatherpop = pops.get(rndIndex(POPULATIONNUMBER));
				//	type = GAHistory.POPXOVER;
					
	//				log.println("Xover between different Populations");
				}
				else
				{
					// Xover within the same Population
					fatherpop = pop;
				//	type = GAHistory.XOVER;
					
	//				log.println("Xover within the same Population");
				}
				
				//search for partner with highest diversity
				int candindex;
				double bestDiff, candDiff;
				IRepresentation cand;
				
				fatherindex = rndIndex(N_ELITE + N_RANDOM);
				father = fatherpop.getNetArray()[fatherindex];
				bestDiff = mother.diffTo(father);
				
				for(int j=0; j<10; j++) {
					candindex = rndIndex(N_ELITE + N_RANDOM);
					cand = fatherpop.getNetArray()[candindex];
					candDiff = mother.diffTo(cand);
	
					if (candDiff > bestDiff) {
						fatherindex=candindex;
						father=cand;
						bestDiff = candDiff;
					}
				}
				
				nets[i] = mother.clone();
				
				//calculate Xover area in chromosom
		/*		int noninodes = config.getIntValue("nn.nodes")-config.getIntValue("nn.inputnodes");
				int nncount = rndIndex(noninodes-1)+1; //minimum 1, maximum all but 1
				int startcpy = config.getIntValue("nn.inputnodes") + rndIndex(noninodes);*/
				
				//create offsprings
				
				boolean isNew=true;
				nets[i].xOverWith(father, XOVERMETHOD);
				//check if this net exist already in the pool
				for (int j=0; j<i; j++)
				{
					if (nets[i].diffTo(nets[j]) == 0) {
						isNew=false;
						//log.println("gen:"+generation+": we found a double");
						break;
					}
				}
				if (isNew==false) {
					i--;	//redo this net
				}
				else {
					//nets[i].getGAHistory().setEntry(generation,type,nets[i].getRank(),nets[i].getScore(),father.getRank());
					//nets[i].setScore(0);					
				}
				
			}
			position += N_XOVER;
		//	System.out.println("\nposition is now "+position);
			pop.readNetArray(nets); //write back netarray to population
		}
		
	}
	
	public void saveResults(Element dfrevo) {
		Element dpopulations = dfrevo.addElement("populations");
		dpopulations.addAttribute("count",String.valueOf(POPULATIONNUMBER));
		dpopulations.addAttribute("generation",String.valueOf(generation));
		dpopulations.addAttribute("randomseed",String.valueOf(this.getSeed()));
		for(SimplePopulation pop:pops)
		{
			pop.exportXml(dpopulations);
		}
	}
	
	private int rndIndex(int range)
	{
		if(range < 1)
			return 0;
		else
			return generator.nextInt(Integer.MAX_VALUE) % range;
	}

	@Override
	public float getProgress() {
		return progress;
	}

	@Override
	public void addResult(File resfile) {
		this.resultfile = resfile;
	}

	@Override
	public ArrayList<ArrayList<IRepresentation>> loadfromxml(Document doc) {
		ArrayList<ArrayList<IRepresentation>> populations = new ArrayList<ArrayList<IRepresentation>>();
		Node dpopulations = doc.selectSingleNode("/frevo/populations");
		//int populationCount = Integer.parseInt(dpopulations.valueOf( "./@count" ));
		List<?> npops = dpopulations.selectNodes(".//population");
		Iterator<?> it = npops.iterator();
		while(it.hasNext())
		{
			Node pop = (Node) it.next();
			ArrayList<IRepresentation> pops = createList(pop);
			populations.add(pops);
		}
		return populations;
	}

}
