package nnga;
/*
 * Copyright (C) 2009 Istvan Fehervari, Wilfried Elmenreich
 * Original project page: http://www.frevotool.tk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License Version 3 as published
 * by the Free Software Foundation http://www.gnu.org/licenses/gpl-3.0.txt
 *
 * There is no warranty for this free software. The GPL requires that 
 * modified versions be marked as changed, so that their problems will
 * not be attributed erroneously to authors of previous versions.
 */
import interfaces.ComponentXMLdata;
import interfaces.IRepresentation;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;

import org.dom4j.Element;


public class SimplePopulation {

	private ArrayList<IRepresentation> members;
	private int popsize;
	private ComponentXMLdata representation;
	private NNGA parent;
	
	public SimplePopulation (int popsize, ComponentXMLdata representation, NNGA p) {
		this.parent = p;
		this.popsize = popsize;
		this.representation = representation;
		createPop();
	}
	
	public SimplePopulation (ArrayList<IRepresentation> candidates, ComponentXMLdata representation, NNGA p) {
		this.parent = p;
		this.representation = representation;
		this.members = candidates;
	}
	
	/** Creates initial population*/
	private void createPop() {
		members = new ArrayList<IRepresentation>();
		for (int i = 0;i<popsize;i++) {
			try {
				IRepresentation member = (IRepresentation)(representation.prepareComponent());
				member.generate (parent.input,parent.output,parent.getRandom());
				members.add(member);
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		}
		
	}
	
	public IRepresentation[] getNetArray()
	{
		IRepresentation[] n = new IRepresentation[popsize];
		return members.toArray(n);
	}
	
	public void rank()
	{
		Collections.sort(members,Collections.reverseOrder());
		
		for (int i=0; i<this.popsize; i++)
			members.get(i).setRank(i);
	}

	public ArrayList<IRepresentation> getMembers() {
		return this.members;
	}

	public IRepresentation genNewNetwork() throws InstantiationException, IllegalAccessException {
		IRepresentation member = (IRepresentation)(representation.prepareComponent());
		member.generate (parent.input,parent.output,parent.getRandom());
		return member;
	}

	public void readNetArray(IRepresentation[] nets) {
		this.members = new ArrayList<IRepresentation>(Arrays.asList(nets));
	}

	public Element exportXml(Element element)
	{
		Element dpop = element.addElement("population");
	/*	dpop.addAttribute("populationsize",String.valueOf(this.popsize));
		dpop.addAttribute("best-score", String.valueOf(this.getBestScore()));
		dpop.addAttribute("avg-score", String.valueOf(this.getAvgPopScore()));*/
		for(IRepresentation n: members)
			n.exportXml(dpop);
		return dpop;
	}
}
