package cam;
/*
 * Copyright (C) 2009 Istvan Fehervari, Wilfried Elmenreich
 * Original project page: http://www.frevotool.tk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License Version 3 as published
 * by the Free Software Foundation http://www.gnu.org/licenses/gpl-3.0.txt
 *
 * There is no warranty for this free software. The GPL requires that 
 * modified versions be marked as changed, so that their problems will
 * not be attributed erroneously to authors of previous versions.
 */
import interfaces.IProblem;
import interfaces.IRepresentation;
import interfaces.PropEntry;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;


public class Cam extends IProblem {
	
	// Input models
	/** Input model where only the Von Neumann environment is considered two times: first the color then the cell output */
	public static final int FOURPLUSFOURVNEUMANNINPUT = 1;
	/** Input model where all 8 neighbors are considered but only their colors */
	public static final int COLORMOOREINPUT = 2;
	/** Input model where all 8 neighbors are considered but only their output */
	public static final int OUTPUTMOOREINPUT = 3;

	private static String imageName=null;
	private static BufferedImage iconImg=null;
	private static int WIDTH;
	private static int HEIGHT;
	private static ArrayList<Integer> colArr = new ArrayList<Integer>();
	private Cell[][] pool;
	public int SIMSTEPS;
	public boolean bestsolutionOutOfNSteps;
	public boolean smartFitnessModel;
	public int fitness_steps;
	public static int COLORS;
	public static final int SCALE = 10;
	//private int bestStep = 0;
	
	IRepresentation[] candidates;

	@Override
	public Hashtable<String, PropEntry> adjustRequirements(
			Hashtable<String, PropEntry> requirements,
			Hashtable<String, PropEntry> properties) {
		// Adjust to inputmodel
		PropEntry inp = properties.get("inputmodel");
		int inputmodel = Integer.parseInt(inp.getValue());
		
		if (inputmodel == OUTPUTMOOREINPUT) {
			PropEntry outputn = requirements.get("outputnumber");
			outputn.setValue(Integer.toString(8));
		}
		
		return requirements;
	}
	
	@Override
	public double getResult(IRepresentation[] candidates) {
		/*
		 * Load image
		 * input: if 1 -> inputs are the colors of the von neumann (4) + 4 inputs for communication
		 * output: own color + 4 output for communication
		 * 
		 */
		
		//load parameters
		this.candidates = candidates;

		SIMSTEPS = Integer.parseInt(getProperties().get("simulation_steps").getValue());
		fitness_steps = Integer.parseInt(getProperties().get("fitness_steps").getValue());
		if (fitness_steps<0) {
			System.err.println("fitness_steps must be at least 1; changed to 1!");
			fitness_steps=1;
		}
		if (fitness_steps>SIMSTEPS) {
			System.err.println("fitness_steps must not be larger than number of simulation steps; changed to "+SIMSTEPS+"!");
			fitness_steps=SIMSTEPS;
		}
		
		bestsolutionOutOfNSteps = Boolean.parseBoolean(getProperties().get("bestsolutionOutOfNSteps").getValue());
		smartFitnessModel =  Boolean.parseBoolean(getProperties().get("smartFitnessModel").getValue());
		//simulate and evaluate
		//long start=System.currentTimeMillis();
		int fitness = calcSim(candidates,SIMSTEPS);
		//System.out.println ("Evaluation of one candidate took "+(System.currentTimeMillis()-start)+"ms");
		//if (fitness > bestFitness) {//what's this good for?
		//	bestFitness=fitness;
//			for (int y =0;y<HEIGHT;y++) {
//				for (int x =0;x<WIDTH;x++) { 
//					System.out.print((char)('a'+pool[x][y].color));
//					System.out.print((char)('A'+pool[x][y].targetcolor));
//					System.out.print(" ");
//				}
//				System.out.println();
//			}
			//System.out.println("fitness:"+fitness);
		//}
		return fitness;
	}

	private int calcFitness() {
		int fitness = 0;
		for (int x =0;x<WIDTH;x++) 
			for (int y =0;y<HEIGHT;y++) {
				int diff = pool[x][y].color - pool[x][y].targetcolor;
				fitness -= diff*diff;
			}
		return fitness;
	}
	
	private int calcWeightedFitness() {
		int fitness = 0;
		for (int x =0;x<WIDTH;x++) 
			for (int y =0;y<HEIGHT;y++) {
				int owndiff=  Math.abs(pool[x][y].color - pool[x][y].targetcolor);
				owndiff = owndiff*owndiff;
				//neighbors, von neumann
				double weight = 0;
				if (x != 0) weight += Math.pow(Math.abs(pool[x][y].targetcolor - pool[x-1][y].targetcolor),2);
				if (x != WIDTH-1) weight += Math.pow(Math.abs(pool[x][y].targetcolor - pool[x+1][y].targetcolor),2);
				if (y != 0) weight += Math.pow(Math.abs(pool[x][y].targetcolor - pool[x][y-1].targetcolor),2);
				if (y != HEIGHT-1) weight += Math.pow(Math.abs(pool[x][y].targetcolor - pool[x][y+1].targetcolor),2);
				
				weight = (weight / Math.pow(COLORS-1,2));
				weight++;
				fitness -= (int)(owndiff * weight);
			}
		return fitness;
	}

	private int calcSim(IRepresentation[] candidates, int steps) {
		
		//Load image only if it is new
		String newImageName = this.getProperties().get("picture").getValue();		
		if (!newImageName.equals(imageName))
			try {
				colArr.clear();
				System.out.println("loading image "+newImageName);
				iconImg = ImageIO.read(new File(newImageName));
				WIDTH = iconImg.getWidth();
				HEIGHT = iconImg.getHeight();
				//Build color list
				for (int x =0;x<WIDTH;x++) 
					for (int y =0;y<HEIGHT;y++) {
						int shortColor=rgbToShortColor(iconImg.getRGB(x, y));
						if (!colArr.contains(shortColor)) {
							colArr.add(shortColor); 
						}
						COLORS = colArr.size();

					}
				Collections.sort(colArr);
				imageName = newImageName;
			} catch (IOException e) {
				e.printStackTrace();
			}

		if (COLORS==0) System.err.println("Error no colors defined!");
		
		//create pool
		pool = new Cell[WIDTH][HEIGHT];
		candidates[0].reset();
		
		int inputmodel = Integer.parseInt(this.getProperties().get("inputmodel").getValue());
		int inputnumber = 4;
		if (inputmodel == FOURPLUSFOURVNEUMANNINPUT) inputnumber = 4;
		else if (inputmodel == COLORMOOREINPUT) inputnumber = 4;
		else if (inputmodel == OUTPUTMOOREINPUT) inputnumber = 8;
		
		for (int x =0;x<WIDTH;x++) {
			for (int y =0;y<HEIGHT;y++) {
				pool [x][y] = new Cell(candidates[0].clone(),colArr.indexOf(rgbToShortColor(iconImg.getRGB(x, y))),inputnumber);
			}
		}
		
		int fitness;
		
		if (bestsolutionOutOfNSteps) 
			fitness = Integer.MIN_VALUE; //for model bestsolutionOutOfNSteps
		else
			fitness = 0; //for model fitness_steps
		
		//int fitness=0;
				
		//Run simulation		
		for (int step = 0; step <steps;step++) {
			for (int x =0;x<WIDTH;x++) {
				for (int y =0;y<HEIGHT;y++) {
					calcCell(inputmodel, x, y);
				}
			}			
			// update new values
			for (int x =0;x<WIDTH;x++) 
				for (int y =0;y<HEIGHT;y++) 
					pool[x][y].update();
			
			int f;
			if (smartFitnessModel)
				f = calcWeightedFitness();
			else
				f = calcFitness();
			
			if (bestsolutionOutOfNSteps) {
				// the best solution within the defined fitness steps defines the fitness
				if (f > fitness) {
					fitness = f;
					//bestStep = step;
				}
			}
			else {
				// fitness is based on the last step
				// if fitness_steps > 1 it is a weighted fitness
				if (steps-step<=fitness_steps) {
					fitness = fitness/2 + f;
				}
			}
		}
		
		return fitness;
	}

	/**
	 * @param inputmodel
	 * @param x
	 * @param y
	 */
	private void calcCell(int inputmodel, int x, int y) {
		Cell c = pool[x][y];
		Vector<Float> input = new Vector<Float>();
		
		if (inputmodel==FOURPLUSFOURVNEUMANNINPUT) {
			//assemble neighbor color inputs
			
			if (x>0)  //east
				input.add((float)pool[x-1][y].color / COLORS);
			else
				input.add(0.0f);
			
			if (x<WIDTH-1)  //west
				input.add((float)pool[x+1][y].color / COLORS);
			else
				input.add(0.0f);						
			
			if (y>0)  //north
				input.add((float)pool[x][y-1].color / COLORS);
			else
				input.add(0.0f);
			
			if (y<HEIGHT-1)  //south
				input.add((float)pool[x][y+1].color / COLORS);
			else
				input.add(0.0f);	
			
			//assemble neighbor color inputs
			
			if (x>0)  //east
				input.add((float)pool[x-1][y].output[3]);
			else
				input.add(0.0f);
			
			if (x<WIDTH-1)  //west
				input.add((float)pool[x+1][y].output[1]);
			else
				input.add(0.0f);						
			
			if (y>0)  //north
				input.add((float)pool[x][y-1].output[2]);
			else
				input.add(0.0f);
			
			if (y<HEIGHT-1)  //south
				input.add((float)pool[x][y+1].output[0]);
			else
				input.add(0.0f);						
		} else if (inputmodel == COLORMOOREINPUT) {
			if (x>0)  //east
				input.add((float)pool[x-1][y].color / COLORS);
			else
				input.add(0.0f);
			
			if (x<WIDTH-1)  //west
				input.add((float)pool[x+1][y].color / COLORS);
			else
				input.add(0.0f);						
			
			if (y>0)  //north
				input.add((float)pool[x][y-1].color / COLORS);
			else
				input.add(0.0f);
			
			if (y<HEIGHT-1)  //south
				input.add((float)pool[x][y+1].color / COLORS);
			else
				input.add(0.0f);
			
			if ((x>0) && (y>0)) { //northeast
				input.add((float)pool[x-1][y-1].color / COLORS);
			} else
				input.add(0.0f);
			
			if ((x<WIDTH-1) && (y>0)) { //northwest
				input.add((float)pool[x+1][y-1].color / COLORS);
			} else
				input.add(0.0f);
			
			if ((x>0) && (y<HEIGHT-1)) { //southeast
				input.add((float)pool[x-1][y+1].color / COLORS);
			} else
				input.add(0.0f);
			
			if ((x<WIDTH-1) && (y<HEIGHT-1)) { //southwest
				input.add((float)pool[x+1][y+1].color / COLORS);
			} else
				input.add(0.0f);			
		} else if (inputmodel == OUTPUTMOOREINPUT) {
			if (x>0)  //east
				input.add((float)pool[x-1][y].output[3]);
			else
				input.add(0.0f);
			
			if (x<WIDTH-1)  //west
				input.add((float)pool[x+1][y].output[1]);
			else
				input.add(0.0f);						
			
			if (y>0)  //north
				input.add((float)pool[x][y-1].output[2]);
			else
				input.add(0.0f);
			
			if (y<HEIGHT-1)  //south
				input.add((float)pool[x][y+1].output[0]);
			else
				input.add(0.0f);
			
			if ((x>0) && (y>0)) { //northeast
				input.add((float)pool[x-1][y-1].output[4]);
			} else
				input.add(0.0f);
			
			if ((x<WIDTH-1) && (y>0)) { //northwest
				input.add((float)pool[x+1][y-1].output[5]);
			} else
				input.add(0.0f);
			
			if ((x>0) && (y<HEIGHT-1)) { //southeast
				input.add((float)pool[x-1][y+1].output[6]);
			} else
				input.add(0.0f);
			
			if ((x<WIDTH-1) && (y<HEIGHT-1)) { //southwest
				input.add((float)pool[x+1][y+1].output[7]);
			} else
				input.add(0.0f);
		}
		
		Vector<Float> output;
		
		output = c.representation.getOutput(input);
		
		
		//first output is color
		//c.nextcolor = (int)((output.firstElement()+1.0)/2.0 *COLORS);
		c.nextcolor = (int)(output.firstElement() *COLORS);
		
		if (c.nextcolor < 0) c.nextcolor=0;
		if (c.nextcolor >= COLORS) c.nextcolor=COLORS-1;
		//next 4 outputs are the connectors
		for (int i=0; i<4; i++)
			c.nextoutput[i]=output.get(i+1);
	}
	
	public static int rgbToShortColor(int rgbcolor)
	{
		int[] rgbs = new int[3];
		int colorDepth=24;
		
		rgbs[0] = (rgbcolor & 0xFF0000) >> 16; // get red
		rgbs[1] = (rgbcolor & 0x00FF00) >> 8; // get green
		rgbs[2] = (rgbcolor & 0x0000FF);  // get blue
		int bitpos = 0x100; //start with bit 8
		int shortColor=0;
		
		for(int i=0; i<colorDepth; i++) {
			if (i % 3 == 0) bitpos >>= 1;
			if ((rgbs[i % 3] & bitpos) > 0)
				shortColor = (shortColor<<1) + 1;
			else
				shortColor <<= 1;
		}
		//System.out.printf("color was %x, converted to %x (%d)\n",rgbcolor,shortColor,shortColor);
		return shortColor;
	}
	
	public static Color shortColorToRGB(int shortColor)
	{
		int colorDepth=24;
		Color rgbcolor=Color.black;
		int[] rgbs = new int[3];
		int bitpos = 1 << (colorDepth-1);
		int bitpos2 = 0x100; //start with bit 8
		
		for(int i=0; i<colorDepth; i++) {
			if (i % 3 == 0) bitpos2 >>= 1;
			if ((shortColor & bitpos) > 0)
				rgbs[i % 3] = rgbs[i % 3] | bitpos2;
			bitpos >>= 1;
		}	
		rgbcolor=new Color(rgbs[0], rgbs[1], rgbs[2]);
		
		return rgbcolor;
	}

	@Override
	public void replayWithVisualization(IRepresentation[] candidates) {
		Cam.this.getResult(candidates);
		Picturepanel picturepanel = new Picturepanel(iconImg,candidates[0]);
		picturepanel.setVisible(true);
	}
	
	private class Picturepanel extends JFrame {
		private static final long serialVersionUID = -4380283761101925684L;
		private BufferedImage referenceimage;
		private WhiteBoard whiteboard;
		JRadioButton computedButton;
		JRadioButton referenceButton;
		IntegerTextField phaseTextField;
		
		public Picturepanel(BufferedImage ref, IRepresentation rep) {
			super ("CAM");
			referenceimage = ref;
			this.setLocationRelativeTo(null);
			int width=this.referenceimage.getWidth()*SCALE+140;
			if (width<250) width=250;
			this.setSize(width, this.referenceimage.getHeight()*SCALE+80);
			setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			this.setLayout(null);
			whiteboard = new WhiteBoard(referenceimage);
			whiteboard.setSize(referenceimage.getWidth()*SCALE, referenceimage.getHeight()*SCALE);
			whiteboard.setLocation(0, 30);
			whiteboard.setBackground(Color.WHITE);
			this.add(whiteboard);
			
			//add radio buttons
			referenceButton = new JRadioButton("Reference");
			referenceButton.setSelected(false);
			referenceButton.setLocation(0, 0);
			referenceButton.setSize(90, 20);
			referenceButton.addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					whiteboard.repaint();
				}
			});
			this.add(referenceButton);
			
			computedButton = new JRadioButton("Evolved");
			computedButton.setSelected(true);
			computedButton.setLocation(90, 0);
			computedButton.setSize(70, 20);
			computedButton.addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					whiteboard.repaint();
				}
			});
			this.add(computedButton);
			
			//add radiogroup
			ButtonGroup group = new ButtonGroup();
		    group.add(referenceButton);
		    group.add(computedButton);

		    //phase button backwards
		    JButton minusButton = new JButton ();
		    minusButton.setLocation(164, 1);
		    minusButton.setSize(20,19);
		    minusButton.addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					SIMSTEPS = Integer.parseInt(phaseTextField.getText());
					if (SIMSTEPS>0) {
						SIMSTEPS--;
						Cam.this.calcSim(candidates, SIMSTEPS);
						whiteboard.repaint();
						phaseTextField.setText(Integer.toString(SIMSTEPS));
					}
				}
			});
		    this.add(minusButton);		    
		    
		    //phase field
		    phaseTextField = new IntegerTextField();
		    phaseTextField.setSize(30, 20);
		    phaseTextField.setLocation(185, 1);
		    phaseTextField.setText(Integer.toString(SIMSTEPS));
		    phaseTextField.addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					SIMSTEPS = Integer.parseInt(phaseTextField.getText());
					int fitness = Cam.this.calcSim(candidates, SIMSTEPS);
					System.out.println ("Weighted fitness :"+fitness+" unweighted fintess: "+calcFitness());
					whiteboard.repaint();
				}
			});

		    this.add(phaseTextField);
		    //phase buttons
		    JButton plusButton = new JButton ();
		    plusButton.setLocation(215, 1);
		    plusButton.setSize(20,19);
		    plusButton.addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					SIMSTEPS = Integer.parseInt(phaseTextField.getText());
					SIMSTEPS++;
					Cam.this.calcSim(candidates, SIMSTEPS);
					whiteboard.repaint();
					phaseTextField.setText(Integer.toString(SIMSTEPS));
				}
			});
		    this.add(plusButton);
		}
		
		private class WhiteBoard extends JPanel {
			private static final long serialVersionUID = 5587457222315758555L;
			private BufferedImage referenceimage;
			
			public WhiteBoard (BufferedImage ref) {
				this.referenceimage = ref;
			}
			public void paintComponent(Graphics g) {
		         super.paintComponent(g);
		         if (referenceButton.isSelected()) {
		        	//draw reference image
			         for (int x = 0;x<this.referenceimage.getWidth();x++) {
			        	 for (int y = 0;y<this.referenceimage.getHeight();y++) {
			        		 g.setColor(new Color (referenceimage.getRGB(x, y)));
			        		 g.fillRect(x*SCALE, y*SCALE, SCALE, SCALE);
			        		 
			        	 }
			         }
		         }
		         else {
		        	//draw computed image		         
			         for (int x = 0;x<this.referenceimage.getWidth();x++) {
			        	 for (int y = 0;y<this.referenceimage.getHeight();y++) {		        		 
			        		 g.setColor(shortColorToRGB(colArr.get(pool[x][y].color)));
			        		 g.fillRect(x*SCALE, y*SCALE, SCALE, SCALE);
			        	 }
			         }
		         }	         
			}
		}
	}
	
	class IntegerTextField extends JTextField {
		private static final long serialVersionUID = -6925002221857070184L;
		final static String badchars 
	       = "`~!@#$%^&*()_+=\\|\"':;?/>.<, ";

	    public void processKeyEvent(KeyEvent ev) {

	        char c = ev.getKeyChar();

	        if((Character.isLetter(c) && !ev.isAltDown()) 
	           || badchars.indexOf(c) > -1) {
	            ev.consume();
	            return;
	        }
	        if(c == '-' && getDocument().getLength() > 0) 
	ev.consume();
	        else super.processKeyEvent(ev);

	    }
	}

}
