package cipi;
/*
 * Copyright (C) 2009 Istvan Fehervari, Wilfried Elmenreich
 * Original project page: http://www.frevotool.tk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License Version 3 as published
 * by the Free Software Foundation http://www.gnu.org/licenses/gpl-3.0.txt
 *
 * There is no warranty for this free software. The GPL requires that 
 * modified versions be marked as changed, so that their problems will
 * not be attributed erroneously to authors of previous versions.
 */
import interfaces.IProblem;
import interfaces.IRepresentation;
import interfaces.PropEntry;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

import MultiSort.MultiSort;

/** Implementation of the Common interest vs. private interest problem */
public class CIPI extends IProblem {
	
	private boolean log = false;
	private BufferedWriter out;
	
	@Override
	public Hashtable<String, PropEntry> adjustRequirements(
			Hashtable<String, PropEntry> requirements,
			Hashtable<String, PropEntry> properties) {
		
		return requirements;
	}

	@Override
	public double getResult(IRepresentation[] candidates) {
		throw new Error ("Please call getMoreResult to evaluate this set.");
	}
	
	public float[] getMoreResult(IRepresentation[] candidates){
		try {
			out = new BufferedWriter(new FileWriter("cipi-log.txt",true));
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		//fix player number now, if changed please call adjust requirements
		int playernumber = 6;
		if (candidates.length != playernumber) throw new Error ("Number candidates should be exactly "+playernumber+"!");
		//Retrieve properties
		PropEntry et = getProperties().get("evaluation_times");
		int evaltimes = Integer.parseInt(et.getValue());

		float[] money = new float[playernumber];//array for storing temporary money
		float[] gainedmoney = new float[playernumber];//array for storing the gained money
		
		for (int i=0;i<playernumber;i++) {
			gainedmoney[i] = 0;
		}
		
		Vector<Float> previousaction = new Vector<Float>();
		//handcraft previous action first
		previousaction.clear();
	//	previousaction.add(1.0f); //indicates that this is a starting action
		for (int i1=0;i1<playernumber;i1++) { //add zero as initial previous action about everyone
			previousaction.add(0f);
		}
		if (log) Log("--------------------------------------");
//		if (log) Log("Players: "+candidates[0]+" "+candidates[1]+" "+candidates[2]+" "+candidates[3]+" "+candidates[4]+" "+candidates[5]);
		//Games
		for (int eval =0;eval<evaltimes;eval++) {
			if (log) Log ("Game "+(eval+1));
			//reset player money
			for (int i=0;i<playernumber;i++) {
				money[i] = 20;
			}
			
			Vector<Float> nextaction = new Vector<Float>();
			nextaction.clear();
			//feed input (action of players from the last round) and collect output
			for (int i=0; i<playernumber; i++) {
				//interpret previous action and put it into nowaction
				Vector<Float> nowaction = new Vector<Float>();
				//add starting indicator
				if (eval==0) nowaction.add(1.0f);
				else nowaction.add(0.0f);
				//add own previous action
				nowaction.add(previousaction.get(i));
				for (int j=0;j<playernumber;j++) {
					if (j!=i) nowaction.add(previousaction.get(j));
				}
				
				candidates[i].reset(); //maybe this does not need
				candidates[i].getOutput(nowaction);
				float action = candidates[i].getOutput(nowaction).get(0);
				
				if (action > money[i]) action = money[i];
				else if (action < 0) action = 0;
				nextaction.add(action);
			}
			if (log) Log ("Donations: "+nextaction.get(0)+" "+nextaction.get(1)+" "+nextaction.get(2)+" "
					+nextaction.get(3)+" "+nextaction.get(4)+" "+nextaction.get(5));
			//execute actions
			float pot = 0;
			for (int i=0; i<playernumber; i++) {
				float donation = nextaction.get(i);
				money[i] = money[i] - donation; //deduct money
				pot = pot + donation; //add money to pot
			}
			
			if (MultiSort.over80) {
				pot *= 3.0;
			}
			else {
				pot *= (pot/20);
			}
			//pot *= 3.0;
			
			
			//redistribute money
			for (int i=0; i<playernumber; i++) {
				money[i] += pot/playernumber;
			}
			if (log)  Log ("Gain: " + (money[0]-20)+" "+ (money[1]-20)+" "+ (money[2]-20)+" "+ (money[3]-20)+" "
					+ (money[4]-20)+" "+ (money[5]-20));
			previousaction = nextaction;
			
			//save only the collected or lost collected money in this turn
			for (int i=0; i<playernumber; i++) {
				gainedmoney[i] = gainedmoney[i]+money[i]-20;
			}
		}	
		if (log) Log ("At the end: "+gainedmoney[0]+" "+gainedmoney[1]+" "+gainedmoney[2]+" "+gainedmoney[3]+" "+gainedmoney[4]+" "+gainedmoney[5]);
		try {
			out.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return gainedmoney;
	}
	
	private void Log(String s) {
		try {
			out.write(s+"\n");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void replayWithVisualization(IRepresentation[] candidates) {
		log = true;
		getMoreResult(candidates);
	}

}
