package light;
import interfaces.IProblem;
import interfaces.IRepresentation;

import java.awt.AWTException;
import java.io.IOException;
import java.util.List;
import java.util.Random;

import javax.swing.SwingWorker;

import utils.ScreenImage;

public class Light extends IProblem {
	
	public IRepresentation nnetwork;
	public boolean withmonitor = false;
	/** The maximum time allowed for the drones to accomplish the mission */
	public static int MAXTIME;
	/** Number of times the simulation is executed */
	public static int EVALUATIONNUMBER;
	/** Real time between redrawing visualization */
	public static final int DISPLAYWAIT = 100;
	public Robot robot;
	public int aktStep;
	public boolean isRunning = false;
	private boolean saveMovie = false;
	public int stepnumber;
	protected DisplayWorker sw;
	private SimDisplay display;
	public SimServer simserver;
	
	private void loadParameters() {
		MAXTIME = Integer.parseInt(getProperties().get("simulationtime").getValue());
		EVALUATIONNUMBER = Integer.parseInt(getProperties().get("evalnumber").getValue());
	}

	@Override
	public double getResult(IRepresentation[] candidates) {
		loadParameters();
		
		this.nnetwork = candidates[0];
		
		return runSimulation(false);
	}

	@Override
	public void replayWithVisualization(IRepresentation[] candidates) {
		loadParameters();
		this.nnetwork = candidates[0];
		
		display = new SimDisplay(this);
	}
	
	private int runSimulation (boolean withmonitor) {
		this.withmonitor = withmonitor;
		if (withmonitor) {
			EVALUATIONNUMBER = 1;
		}
		
		int fitness = 0;
		if (this.withmonitor) isRunning = true;
		
		//Iterate through
		for (int n = 0; n < EVALUATIONNUMBER; n++) {
			//create robot
			this.robot = new Robot(new RobotController(nnetwork,this));
			robot.setBodyDirection(generator.nextInt(360));
		
			//initialize time
			stepnumber = MAXTIME / (int)SimServer.STEPLENGTH;
			// create new simulation environment
			if (withmonitor) this.simserver = new SimServer(this,Long.parseLong(display.seedField.getText()));

			else {
				this.simserver = new SimServer(this, new Random().nextLong());
			}
			
			if (withmonitor) {
				sw.setProgressToPublish(aktStep);
			}
			if (saveMovie) saveFrame();		
			/* Simulator is ready */
			
			try {
				calculateStep();
			} catch (Exception e) {
				e.printStackTrace();
			}
			
			if (withmonitor)
				pause(DISPLAYWAIT); //pause 100 it was
			for (aktStep = 0; aktStep < stepnumber; aktStep++) {
				
				if (saveMovie) saveFrame();
				calculateStep();
				if (withmonitor)
					pause(DISPLAYWAIT); //pause 100 it was
				if (simserver.succeeded) {
					break;
				}
				if ((withmonitor) && (isRunning == false)) break;
			}
			
			//get fitness
			fitness += (-aktStep-SimServer.getDistance(robot.getPosition(), simserver.targetlocation));
			if (withmonitor) System.out.println ("Fitness: "+fitness);
		}
		if (withmonitor) display.startButton.setEnabled(true);
		isRunning = false;
		return fitness/EVALUATIONNUMBER;
		
	}
	
	public void calculateStep() {
		//send sensor data
		sendSensordata();
		
		// get new intentions
		robot.getController().postInfo();
		
		//calculate changes
		simserver.calculateAll();

		if (withmonitor) sw.setProgressToPublish(aktStep);
		
	}
	
	/** Sends sensor data to the drones */
	public void sendSensordata() {
		// reset controllers
		robot.getController().preInfo();
		
		//Add visual information
		simserver.getVisuals(robot);
	}
	
	private void saveFrame() {
		try {
			ScreenImage.createImage(display, "./Images/Frame_"+fix(aktStep)+".bmp");
		} catch (AWTException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}
	
	private class DisplayWorker extends SwingWorker<Void,Integer> {

		@Override
		protected Void doInBackground() throws Exception {
			//loadParameters();						
			runSimulation(true);
			return null;
		}
		
		public void setProgressToPublish (int p) {
			publish(p);
	    }
		
	    protected void process(List<Integer> results) {
	    	display.updateDisplay();
	    }
		
	}
	
	private String fix(int num) {
		String ret = Integer.toString(num);
		if (num < 10) ret = "00"+ret;
		else if (num < 100) ret = "0"+ret;
		return ret;
	}
	
	protected synchronized void pause (int ms) {
	    try {
	      this.wait(ms);
	    } catch (InterruptedException ex) {
	    }
	}
	
	public void runBackground() {
		sw = new DisplayWorker();
		sw.execute();
	}

}
