package light;
import graphics.FrevoWindow.IntegerTextField;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Random;

import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;


public class SimDisplay extends JFrame {

	private static final long serialVersionUID = -3845311699398981320L;
	private JPanel menuPanel;
	private DrawPanel canvasPanel;

	public IntegerTextField seedField;
	
	private Random generator = new Random();
	
	private Icon playIcon;
	private Icon stopIcon;
	
	protected JButton startButton;
	protected JButton stopButton;
	
	public static final Color FIELDCOLOR = new Color (201,196,169);
	
	private Light master;
	
	public SimDisplay(Light master) {
		super("2D Robot Light! Simulation");
	    this.master = master;
	    setBounds(0,0,620,688);// set initial frame
	    setMinimumSize(new Dimension (320,320));
	    setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
	    Container con = this.getContentPane(); // inherit main frame
	    con.setLayout(new BoxLayout(con, BoxLayout.Y_AXIS));
	    
	    //Create Menu frame
	    menuPanel = new JPanel();
	    menuPanel.setPreferredSize(new Dimension(300, 50));
	    menuPanel.setMinimumSize(new Dimension(300, 50));
	    menuPanel.setMaximumSize(new Dimension(300, 50));
	    menuPanel.setBorder(new TitledBorder("Control"));
	    menuPanel.setLayout(new BoxLayout(menuPanel, BoxLayout.X_AXIS));
	    
	    startButton = new JButton ();
	    startButton.setSize(30, 25);
	    menuPanel.add(startButton);
	    
	    stopButton = new JButton ();
		stopButton.setSize(30, 25); 
	    menuPanel.add(stopButton);
	    
	    try {
			playIcon = new ImageIcon(new URL("jar:file:./Libraries/jlfgr-1_0.jar/!" + "/" + "toolbarButtonGraphics/media/Play24.gif"));
			stopIcon = new ImageIcon(new URL("jar:file:./Libraries/jlfgr-1_0.jar/!" + "/" + "toolbarButtonGraphics/media/Stop24.gif"));
			startButton.setIcon(playIcon);
			stopButton.setIcon(stopIcon);
		} catch (MalformedURLException e1) {
			e1.printStackTrace();
		}
				
	    startButton.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent arg0) {							
				SimDisplay.this.master.runBackground();
				startButton.setEnabled(false);
			}
		});
	    
	    stopButton.addActionListener(new ActionListener() {			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				if (SimDisplay.this.master.isRunning) {
					SimDisplay.this.master.isRunning = false;
				}
			}
		});
	    
	    JLabel seedLabel = new JLabel ("Seed:");
	    seedLabel.setPreferredSize(new Dimension(50, 30));
	    seedLabel.setMinimumSize(new Dimension(50, 30));
	    seedLabel.setMaximumSize(new Dimension(50, 30));
	    menuPanel.add(seedLabel);
	    
	    seedField = new IntegerTextField();
	    seedField.setPreferredSize(new Dimension(40, 20));
	    seedField.setMinimumSize(new Dimension(40, 20));
	    seedField.setMaximumSize(new Dimension(40, 20));
	    seedField.setText(Integer.toString(1234));
	    menuPanel.add(seedField);
	    
	    JButton randomButton = new JButton ("Random");
	    randomButton.setSize(90, 25);
	    randomButton.setLocation(160, 18);
	    randomButton.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				seedField.setText(Integer.toString(generator.nextInt(10000)));
			}
		});
	    menuPanel.add(randomButton);
	    
	    con.add(menuPanel);
	    
	    canvasPanel = new DrawPanel();
	    
	    con.add(canvasPanel);
	    
	    setVisible(true);  // add to frame and show
	    canvasPanel.repaint();
	    canvasPanel.setBackground(FIELDCOLOR);
	    
	    canvasPanel.addComponentListener(new ComponentListener() {
			
			@Override
			public void componentShown(ComponentEvent e) {
			}
			
			@Override
			public void componentResized(ComponentEvent e) {
				e.getComponent().repaint();
			}
			
			@Override
			public void componentMoved(ComponentEvent e) {
			}
			
			@Override
			public void componentHidden(ComponentEvent e) {
			}
		});
	    
	    this.setLocationRelativeTo(null);
	}
	public void updateDisplay() {
		canvasPanel.repaint();
		this.repaint();
	}
	
	class DrawPanel extends JPanel
	{
		private static final long serialVersionUID = -8748735749891520153L;
		
		public void paintComponent(Graphics g) {
			super.paintComponent(g);
			//calculate scaling property
			int w = getWidth()-20;
			int h = getHeight()-20;//leave some border, draw inside
			double prop;
			Point origo;//middle point of inner
			int u =0;
			if (w<=h) u = w;
			else u = h;
			
			prop = (double)(u) / (double)(SimServer.FIELDDIM);
			origo = new Point (10+u/2,10+u/2);
			
			//draw field boundary
			g.setColor(Color.BLACK);
			g.drawLine(10,10, u, 10);
			g.drawLine(10, u, u, u);
			g.drawLine(10, 10, 10, u);
			g.drawLine(u, 10, u, u);
			
			//Draw rest only if parent object exists			
			if (master.simserver != null) {
								
				//draw target range circle
				int rad = (int)(SimServer.LIGHTSOURCE_PLACEMENT_RADIUS*prop);
				g.setColor(Color.RED);

				for (int i=0;i<18;i++)	g.drawArc(origo.x-rad, origo.y-rad, rad*2, rad*2, i*20, 10);
				//draw target
				Point tloc = new Point ((int)(master.simserver.targetlocation.x* prop+origo.x),(int)(master.simserver.targetlocation.y* prop+origo.y) );
				int ldia = (int)(SimServer.LIGHTSOURCE_SIZE*prop);
				g.setColor(Color.WHITE);
				g.drawOval(tloc.x-ldia/2, tloc.y-ldia/2, ldia, ldia);
			/*	
				g.fillPolygon(new int[]{tloc.x,tloc.x+ldia/2,tloc.x,tloc.x-ldia/2}, new int[]{tloc.y-ldia/2,tloc.y,tloc.y+ldia/2,tloc.y}, 4);
				g.setColor(new Color (245,255,135));
				g.fillPolygon(new int[]{tloc.x,tloc.x+ldia/2-3,tloc.x,tloc.x-ldia/2+3}, new int[]{tloc.y-ldia/2+3,tloc.y,tloc.y+ldia/2-3,tloc.y}, 4);
	*/
				//draw robot
				g.setColor(Robot.BASECOLOR);
				Point robotcenter = new Point ((int)(master.robot.getPosition().x* prop+origo.x),(int)(master.robot.getPosition().y	*prop +origo.y));
				//draw a point always
				g.drawLine(robotcenter.x, robotcenter.y, robotcenter.x, robotcenter.y);

				//draw body
				g.fillOval((int)(robotcenter.x-(Robot.ROBOT_DIAMETER*prop/2.0)), (int)(robotcenter.y-(Robot.ROBOT_DIAMETER*prop/2.0)), (int)(Robot.ROBOT_DIAMETER*prop), (int)(Robot.ROBOT_DIAMETER*prop));
				
				//draw robot orientation
				g.setColor(Color.RED);
				double dir = Math.toRadians(master.robot.getBodyDirection());		
				g.drawLine(robotcenter.x, robotcenter.y, (int)(robotcenter.x+Math.sin(dir)*(Robot.ROBOT_DIAMETER*prop/2.0)),(int)( robotcenter.y-Math.cos(dir)*(Robot.ROBOT_DIAMETER*prop/2.0)));
				
				//draw wheels
				g.setColor(Robot.WHEELCOLOR);
				
				Point leftwheelcenter = new Point ((int)(robotcenter.x+Math.sin(dir-1.57)*(Robot.ROBOT_DIAMETER*prop/2.0)),(int)( robotcenter.y-Math.cos(dir-1.57)*(Robot.ROBOT_DIAMETER*prop/2.0)));
				Point leftwheelcenter2 = new Point ((int)(robotcenter.x+Math.sin(dir-1.57)*((Robot.ROBOT_DIAMETER*prop/2.0)-1)),(int)( robotcenter.y-Math.cos(dir-1.57)*((Robot.ROBOT_DIAMETER*prop/2.0)-1)));
				
				g.drawLine((int)(leftwheelcenter.x+Math.sin(dir)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)), (int)( leftwheelcenter.y-Math.cos(dir)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)),(int)(leftwheelcenter.x+Math.sin(dir+3.14)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)), (int)( leftwheelcenter.y-Math.cos(dir+3.14)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)));				
				g.drawLine((int)(leftwheelcenter2.x+Math.sin(dir)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)), (int)( leftwheelcenter2.y-Math.cos(dir)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)),(int)(leftwheelcenter2.x+Math.sin(dir+3.14)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)), (int)( leftwheelcenter2.y-Math.cos(dir+3.14)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)));
				
				Point rightwheelcenter = new Point ((int)(robotcenter.x+Math.sin(dir+1.57)*(Robot.ROBOT_DIAMETER*prop/2.0)),(int)( robotcenter.y-Math.cos(dir+1.57)*(Robot.ROBOT_DIAMETER*prop/2.0)));
				Point rightwheelcenter2 = new Point ((int)(robotcenter.x+Math.sin(dir+1.57)*((Robot.ROBOT_DIAMETER*prop/2.0)-1)),(int)( robotcenter.y-Math.cos(dir+1.57)*((Robot.ROBOT_DIAMETER*prop/2.0)-1)));

				g.drawLine((int)(rightwheelcenter.x+Math.sin(dir)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)), (int)( rightwheelcenter.y-Math.cos(dir)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)),(int)(rightwheelcenter.x+Math.sin(dir+3.14)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)), (int)( rightwheelcenter.y-Math.cos(dir+3.14)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)));				
				g.drawLine((int)(rightwheelcenter2.x+Math.sin(dir)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)), (int)( rightwheelcenter2.y-Math.cos(dir)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)),(int)(rightwheelcenter2.x+Math.sin(dir+3.14)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)), (int)( rightwheelcenter2.y-Math.cos(dir+3.14)*(Robot.ROBOT_WHEEL_DIAMETER*prop/2.0)));
				
				//draw light sensor range
				g.setColor(Color.ORANGE);
				
				g.drawLine (leftwheelcenter.x,leftwheelcenter.y,(int)(leftwheelcenter.x+Math.sin(dir-Math.toRadians(Robot.LIGHT_SENSOR_ANGLE/2))*(Robot.LIGHT_SENSOR_RANGE*prop)),(int)( leftwheelcenter.y-Math.cos(dir-Math.toRadians(Robot.LIGHT_SENSOR_ANGLE/2))*(Robot.LIGHT_SENSOR_RANGE*prop)));
				g.drawLine (leftwheelcenter.x,leftwheelcenter.y,(int)(leftwheelcenter.x+Math.sin(dir+Math.toRadians(Robot.LIGHT_SENSOR_ANGLE/2))*(Robot.LIGHT_SENSOR_RANGE*prop)),(int)( leftwheelcenter.y-Math.cos(dir+Math.toRadians(Robot.LIGHT_SENSOR_ANGLE/2))*(Robot.LIGHT_SENSOR_RANGE*prop)));
				g.drawArc((int)(leftwheelcenter.x-Robot.LIGHT_SENSOR_RANGE*prop), (int)(leftwheelcenter.y-Robot.LIGHT_SENSOR_RANGE*prop), (int)(Robot.LIGHT_SENSOR_RANGE*2*prop), (int)(Robot.LIGHT_SENSOR_RANGE*2*prop), (int)(-master.robot.getBodyDirection()+90-(Robot.LIGHT_SENSOR_ANGLE/2)), (int)(Robot.LIGHT_SENSOR_ANGLE));
				
				g.drawLine (rightwheelcenter.x,rightwheelcenter.y,(int)(rightwheelcenter.x+Math.sin(dir+Math.toRadians(Robot.LIGHT_SENSOR_ANGLE/2))*(Robot.LIGHT_SENSOR_RANGE*prop)),(int)( rightwheelcenter.y-Math.cos(dir+Math.toRadians(Robot.LIGHT_SENSOR_ANGLE/2))*(Robot.LIGHT_SENSOR_RANGE*prop)));
				g.drawLine (rightwheelcenter.x,rightwheelcenter.y,(int)(rightwheelcenter.x+Math.sin(dir-Math.toRadians(Robot.LIGHT_SENSOR_ANGLE/2))*(Robot.LIGHT_SENSOR_RANGE*prop)),(int)( rightwheelcenter.y-Math.cos(dir-Math.toRadians(Robot.LIGHT_SENSOR_ANGLE/2))*(Robot.LIGHT_SENSOR_RANGE*prop)));
				g.drawArc((int)(rightwheelcenter.x-Robot.LIGHT_SENSOR_RANGE*prop), (int)(rightwheelcenter.y-Robot.LIGHT_SENSOR_RANGE*prop), (int)(Robot.LIGHT_SENSOR_RANGE*2*prop), (int)(Robot.LIGHT_SENSOR_RANGE*2*prop), (int)(-master.robot.getBodyDirection()+90-(Robot.LIGHT_SENSOR_ANGLE/2)), (int)(Robot.LIGHT_SENSOR_ANGLE));
				
				//highlight sensor if active
				if (master.robot.getController().sensorvalues[0] != 0)
				g.fillArc((int)(leftwheelcenter.x-Robot.LIGHT_SENSOR_RANGE*prop), (int)(leftwheelcenter.y-Robot.LIGHT_SENSOR_RANGE*prop), (int)(Robot.LIGHT_SENSOR_RANGE*2*prop), (int)(Robot.LIGHT_SENSOR_RANGE*2*prop), (int)(-master.robot.getBodyDirection()+90-(Robot.LIGHT_SENSOR_ANGLE/2)), (int)(Robot.LIGHT_SENSOR_ANGLE));
				if (master.robot.getController().sensorvalues[1] != 0) {
					g.fillArc((int)(rightwheelcenter.x-Robot.LIGHT_SENSOR_RANGE*prop), (int)(rightwheelcenter.y-Robot.LIGHT_SENSOR_RANGE*prop), (int)(Robot.LIGHT_SENSOR_RANGE*2*prop), (int)(Robot.LIGHT_SENSOR_RANGE*2*prop), (int)(-master.robot.getBodyDirection()+90-(Robot.LIGHT_SENSOR_ANGLE/2)), (int)(Robot.LIGHT_SENSOR_ANGLE));
					//System.out.println ("Right sensor detect!");
				}
				
			}
			//draw the step number to lower left corner
			g.setColor(Color.BLACK);
			g.drawString("Step: "+master.stepnumber+"/"+Integer.toString(master.aktStep), 15, u-5);
	
		}
		
		// implements double buffering to avoid flickers
	    public void update(Graphics g) {
	    	Graphics offgc;
	    	Image offscreen = null;
	    	Dimension d = getSize();

	    	// create the offscreen buffer and associated Graphics
	    	offscreen = createImage(d.width, d.height);
	    	offgc = offscreen.getGraphics();
	    	// clear the exposed area
	    	offgc.setColor(getBackground());
	    	offgc.fillRect(0, 0, d.width, d.height);
	    	offgc.setColor(getForeground());
	    	// do normal redraw
	    	paint(offgc);
	    	// transfer offscreen to window
	    	g.drawImage(offscreen, 0, 0, this);
	    }
		 
	}
}
