package AbsoluteRanking;
/*
 * Copyright (C) 2009 Istvan Fehervari, Wilfried Elmenreich
 * Original project page: http://www.frevotool.tk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License Version 3 as published
 * by the Free Software Foundation http://www.gnu.org/licenses/gpl-3.0.txt
 *
 * There is no warranty for this free software. The GPL requires that 
 * modified versions be marked as changed, so that their problems will
 * not be attributed erroneously to authors of previous versions.
 */
import interfaces.ComponentXMLdata;
import interfaces.IMethod;
import interfaces.IProblem;
import interfaces.IRanking;
import interfaces.IRepresentation;

import java.util.ArrayList;
import java.util.Collections;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;


public class AbsoluteRanking extends IRanking {

	public static double BESTFITNESS;

	class myRunnable implements Runnable {
		IRepresentation cand;
		ComponentXMLdata problem;
		IMethod method;

		public myRunnable(IRepresentation cand,ComponentXMLdata problem, IMethod method) {
			this.cand = cand;
			this.problem = problem;
			this.method = method;
		}

		@Override
		public void run() {
			try {
				IProblem p = (IProblem)(problem.prepareComponent());
				p.setRandom(method.getRandom().clone());
				cand.reset(); //wipe clean state of the candidate 
				double fitness = p.getResult(new IRepresentation[]{cand});
				cand.setFitness(fitness);
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		}
	}	

	@Override
	public void rank(ArrayList<IRepresentation> pop, ComponentXMLdata problem, IMethod method) {
		int nthreads=Integer.parseInt(getProperties().get("parallelthreads").getValue());

		if (nthreads<=1) {
			//evaluate using for loop
			for (IRepresentation cand:pop) {
				try {
					IProblem p;
					p = (IProblem)(problem.prepareComponent());
					p.setRandom(method.getRandom());
					cand.reset(); //wipe clean state of the candidate 
					double fitness = p.getResult(new IRepresentation[]{cand});
					cand.setFitness(fitness);
				} catch (InstantiationException e) {
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}
			}
		}
		else {
			// evaluate using thread pool
			ExecutorService executor = Executors.newFixedThreadPool(nthreads);
			for (IRepresentation cand:pop) {
				Runnable worker = new myRunnable(cand, problem, method);
				executor.execute(worker);
			}

			// This will make the executor accept no new threads
			// and finish all existing threads in the queue
			executor.shutdown();
			// Wait until all threads are finish
			while (!executor.isTerminated()) {
				try {
					executor.awaitTermination(1, TimeUnit.SECONDS);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}
			// close the pool and wait for all tasks to finish.
			
		}

		//sort with highest fitness first
		Collections.sort(pop, Collections.reverseOrder());
				
		BESTFITNESS = pop.get(0).getFitness();

		//System.out.println("\n");
		//for (int i=0; i<pop.size(); i++) {
		//	System.out.println(i+". "+pop.get(i).getUniqueName()+"\t"+pop.get(i).getFitness());
		//}
		
		System.out.print("Best Agent: "+pop.get(0).getUniqueName()+"\t");
		System.out.println("Best Fitness: "+pop.get(0).getFitness());
		//System.out.println("Worst Fitness: "+pop.get(pop.size()-1).getFitness());
	}

}
