package graphics;

import interfaces.ComponentXMLdata;
import interfaces.PropEntry;
import interfaces.PropTypes;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.GroupLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeSelectionModel;

import main.FrevoMain;
import main.FrevoMain.TagElement;

public class ComponentBrowser extends JFrame implements WindowListener, MouseListener, TableModelListener, ActionListener {

	private static final long serialVersionUID = 590310985776157487L;
	
	private JTree tree;
	private JScrollPane tableScrollPane;
	private JButton confirmButton;
	private JButton cancelButton;
	private JTextPane descrPane;
	JSplitPane topsplitPane;
	JSplitPane bigsplitPane;
	
	private ArrayList<ComponentXMLdata> componentList = new ArrayList<ComponentXMLdata>();
	private Hashtable<String, PropEntry> properties;
	private JTable propTable = new JTable();
	private PropertiesTableModel model;
	private int componentType;
	
	private ImageIcon noimage;
	int SELECTEDITEM = -1;
	
	private Dimension iconSize = new Dimension(36,36);
	
	ArrayList<FrevoMain.TagElement> categories;
	ArrayList<DefaultMutableTreeNode> categorynodes = new ArrayList<DefaultMutableTreeNode>();

	public ComponentBrowser(int componenttype) {
		super ("Select "+FrevoMain.getComponentTypeAsString(componenttype)+ " Component");
		componentType = componenttype;
		categories = FrevoMain.getCategories(componenttype);
		
		setBounds(0,0,FrevoMain.componentBrowserParameters[0],FrevoMain.componentBrowserParameters[1]);// set initial size
		Container con = this.getContentPane(); // inherit main frame
        this.setLocationRelativeTo(null);
        setMinimumSize(new Dimension(500,550));
        
        //load noimage and resize
        noimage = FrevoMain.mainWindow.nothingIcon;
        noimage = resizeImage(noimage,iconSize.width, iconSize.height);  
 
        //return focus to main window after this has been closed
		this.addWindowListener(this);
		
		//Get all components
		componentList.clear();
		for (int i=0;i<FrevoMain.getComponentList(componenttype).size();i++)
			componentList.add(FrevoMain.getComponentList(componenttype).get(i));
        
        //filter components based on other selected components
        filterComponentList();
		
		DefaultMutableTreeNode top =
	        new DefaultMutableTreeNode(FrevoMain.getComponentTypeAsString(componenttype)+"s");
	    createTree(top,componenttype);
	    tree = new JTree(top);
	    tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
	    
	    tree.addMouseListener(this);
	    
	    tree.setShowsRootHandles(false);
		
	    tree.expandRow(tree.getRowCount()-1);
	    
	    //add tree renderer
	    tree.setCellRenderer(new IconTreeCellRenderer());
	    
		JScrollPane treeview = new JScrollPane(tree);
		treeview.setMinimumSize(new Dimension(270,200));
		
		descrPane = new JTextPane();
        descrPane.setEditable(false);
        descrPane.setFont(new Font("Monospaced", 1, 13));
        descrPane.setCaretPosition(0);
        descrPane.setContentType("text/html");
        
        StyledDocument doc= descrPane.getStyledDocument();
        MutableAttributeSet attr= new SimpleAttributeSet();
        StyleConstants.setLineSpacing(attr, -0.2f);
        doc.setParagraphAttributes(0, doc.getLength()-10, attr, false);
		
		JScrollPane descview = new JScrollPane(descrPane);
		descview.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		
		topsplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				treeview, descview);
		topsplitPane.setPreferredSize(new Dimension(500,270));
		topsplitPane.setDividerLocation(FrevoMain.componentBrowserParameters[2]);
		
		tableScrollPane = new JScrollPane();
		tableScrollPane.setPreferredSize(new Dimension(500,170));
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.setMaximumSize(new Dimension(5000,40));
		
		confirmButton = new JButton("Confirm");
		confirmButton.setEnabled(false);
		confirmButton.addActionListener(this);
		
		cancelButton = new JButton("Cancel");
		cancelButton.addActionListener(this);
		
		buttonPanel.add(confirmButton);
		buttonPanel.add(cancelButton);
		
		bigsplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,topsplitPane,tableScrollPane);
		bigsplitPane.setDividerLocation(FrevoMain.componentBrowserParameters[3]);
		
		//define layout		
		GroupLayout mainlayout = new GroupLayout(con);
        con.setLayout(mainlayout);
        
        mainlayout.setHorizontalGroup(mainlayout.createParallelGroup()
        		.addComponent(bigsplitPane)
        		.addComponent(buttonPanel)
        );
        mainlayout.setVerticalGroup(mainlayout.createSequentialGroup()
        		.addComponent(bigsplitPane)
        		.addComponent(buttonPanel)
        );
        
	}
	
	private void filterComponentList() {
		if (this.componentType == FrevoMain.FREVO_IRANKING) {
			//filter rankings based on selected iproblem
			ComponentXMLdata selectedproblem = FrevoMain.iProblems.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IPROBLEM]);
			PropEntry maxplayersEntry = selectedproblem.getRequirements().get("maximumCandidates");
			PropEntry minplayersEntry = selectedproblem.getRequirements().get("minimumCandidates");
			int maxplayers = Integer.parseInt(maxplayersEntry.getValue());
			int minplayers = Integer.parseInt(minplayersEntry.getValue());
			
			if (minplayers > 1) {
				//disable absoluterank
				Iterator<ComponentXMLdata> itr = componentList.iterator();
			    while (itr.hasNext()) {
			    	ComponentXMLdata cdata = itr.next();
			    	if (cdata.getClassName().equals("AbsoluteRanking.AbsoluteRanking")) {
			    		componentList.remove(cdata);
				    	break;
			    	}			    	
			    }
			}
			else if (maxplayers < 2) {
				//disable all tournaments
				Iterator<ComponentXMLdata> itr = componentList.iterator();
			    while (itr.hasNext()) {
			    	ComponentXMLdata cdata = itr.next();
			    	if (!cdata.getClassName().equals("AbsoluteRanking.AbsoluteRanking")) {
			    		itr.remove();
			    	}			    	
			    }
			}
						
		}
	}

	private void saveChanges() {
		//save preferences
		FrevoMain.componentBrowserParameters[0] = ComponentBrowser.this.getBounds().width;
		FrevoMain.componentBrowserParameters[1] = ComponentBrowser.this.getBounds().height;
		FrevoMain.componentBrowserParameters[2] = topsplitPane.getDividerLocation();
		FrevoMain.componentBrowserParameters[3] = bigsplitPane.getDividerLocation();
	}
	
	/**
	 * finishes componentbrowser session
	 */
	private void submitChanges() {
		int oldSelectedComponent = FrevoMain.SELECTED_COMPONENT[componentType];
		FrevoMain.SELECTED_COMPONENT[componentType] = SELECTEDITEM;
		
		saveChanges();
		
		//new iproblem selected
		if ((oldSelectedComponent != SELECTEDITEM) && (componentType == FrevoMain.FREVO_IPROBLEM)) {
			if (FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IRANKING] != -1) {
				//check compatibility with iranking
				ComponentXMLdata selectedproblem = FrevoMain.iProblems
						.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IPROBLEM]);
				PropEntry maxplayersEntry = selectedproblem.getRequirements()
						.get("maximumCandidates");
				PropEntry minplayersEntry = selectedproblem.getRequirements()
						.get("minimumCandidates");
				int maxplayers = Integer.parseInt(maxplayersEntry.getValue());
				int minplayers = Integer.parseInt(minplayersEntry.getValue());
				ComponentXMLdata selectedranking = FrevoMain.iRankings
						.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IRANKING]);
				if (selectedranking.getClassName().equals(
						"AbsoluteRanking.AbsoluteRanking")) {
					if (minplayers > 1) {
						//invalid ranking
						FrevoMain.mainWindow.raBrowser = null;
						FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IRANKING] = -1;

					}
				} else {
					if (maxplayers < 2) {
						//invalid ranking
						FrevoMain.mainWindow.raBrowser = null;
						FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IRANKING] = -1;
					}
				}
			} else {
				FrevoMain.mainWindow.raBrowser = null;
			}
			FrevoMain.customName = FrevoMain.iProblems.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IPROBLEM]).getName();
			FrevoMain.mainWindow.updateAdvancedLabel();
		}

		FrevoMain.mainWindow.setEnabled(true);
		ComponentBrowser.this.setVisible(false);
	}
	
	private class IconTreeCellRenderer extends DefaultTreeCellRenderer {
		private static final long serialVersionUID = -6635134544981447344L;
		public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel,
		                boolean expanded, boolean leaf, int row, boolean hasFocus) {
		
			super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
			
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)value;
					
			//actual component
			if ((leaf) && (node.getUserObject() instanceof ComponentXMLdata  )) {
				ComponentXMLdata cdata = (ComponentXMLdata)(node.getUserObject());			
				String imagepath = (FrevoMain.getComponentDirectory(componentType) + cdata.getClassDir()+"//"+cdata.getImageLocation());		    
				ImageIcon componentIcon = new ImageIcon(imagepath,cdata.getName());
				
				if (componentIcon.getImageLoadStatus() != java.awt.MediaTracker.ERRORED ) {
					setIcon(resizeImage(componentIcon,iconSize.width,iconSize.height));
				}
				else setIcon(noimage);	
			} else if (node.getUserObject() instanceof TagElement) {
				TagElement nodeInfo = (TagElement)node.getUserObject();
				ImageIcon img = new ImageIcon(ClassLoader.getSystemResource("Categories/"+FrevoMain.getComponentTypeAsString(componentType)+"/"+nodeInfo.getImageName() ));
				setIcon(resizeImage(img,iconSize.width,iconSize.height));
			} else {
				//The root icon
				ImageIcon img = new ImageIcon(ClassLoader.getSystemResource("Categories/"+FrevoMain.getComponentTypeAsString(componentType)+"/"+"empty-folder.png"));
				setIcon(resizeImage(img,iconSize.width,iconSize.height));
			}
				
		return this;
		}
	}

	private void createTree(DefaultMutableTreeNode top, int type) {
		//List containing the tags for this component type
		
		int[] elementnumber = new int [categories.size()];
		for (int i=0;i<categories.size();i++) elementnumber[i] = 0;
		
		int unsortednumber = 0;
		
		//add categories
		for (int i=0;i<categories.size();i++) {
			DefaultMutableTreeNode node = new DefaultMutableTreeNode(categories.get(i)); 
			top.add(node);
			categorynodes.add(node);
		}
		
		DefaultMutableTreeNode categoryUnsorted = new DefaultMutableTreeNode(new TagElement("Unsorted",null,"unsorted.png"));
	    		
		DefaultMutableTreeNode categoryAll = new DefaultMutableTreeNode(new TagElement("All",null,"all.png"));
	
	    //sort components into categories
		for (int i=0;i<componentList.size();i++) {//iterate through components
			ComponentXMLdata cdata = componentList.get(i);
			ArrayList<String> ctags = cdata.getTags();
			boolean sorted = false;
			
			for (int k=0;k<categories.size();k++) {//iterate through all categories
				boolean placed = false;
				for (int j=0;j<ctags.size();j++) {//iterate through this components tags
					for (int l=0;l<categories.get(k).getTags().size();l++) {//iterate through all tags of this category
						if (ctags.get(j).contains(categories.get(k).getTags().get(l))) {
							placed = true;
							sorted = true;
						}
					}
				}
				if (placed) {
					//add
					DefaultMutableTreeNode child = (DefaultMutableTreeNode)top.getChildAt(k);
					child.add(new DefaultMutableTreeNode(cdata));
					//adjust category counter
					elementnumber[k]++;
				}				
			}
			if (!sorted) {
				categoryUnsorted.add(new DefaultMutableTreeNode(cdata));
				unsortednumber++;
			}
			//add to "All" category as well
			categoryAll.add(new DefaultMutableTreeNode(cdata));
		}
		
		if (unsortednumber > 0)	top.add(categoryUnsorted);
		
		top.add(categoryAll);
		
		//adjust names to represent number of items
		for (int i=0;i<categories.size();i++) {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)top.getChildAt(i);
			TagElement nInfo = (TagElement)(node.getUserObject());
			nInfo.setElements(elementnumber[i]);
		}
		
		TagElement nodeInfo = (TagElement)(categoryUnsorted.getUserObject());
		nodeInfo.setElements(unsortednumber);
		
		nodeInfo = (TagElement)(categoryAll.getUserObject());
		nodeInfo.setElements(componentList.size());
	}
	
	public static ImageIcon resizeImage(ImageIcon src, int width, int height) {
		Image img = src.getImage();
        Image newimg = img.getScaledInstance(width, height,  java.awt.Image.SCALE_SMOOTH);  
        return new ImageIcon(newimg);
	}
	
	private void loadSelected(ComponentXMLdata cdata) {
		//Load description
		String description = cdata.getDescription();
		descrPane.setText("<center><b>"+cdata.getName()+"</b></center><br><FONT FACE=\"Arial\">"+description+"</font>");
		
		//Load table for data
		//Load properties
		properties = cdata.getProperties();
		//Setup Table
        String headers[] = { "Keys", "Values" };
        model = new PropertiesTableModel(headers,properties.size());
        model.addTableModelListener(this);
        
        propTable = new JTable(model);
        propTable.setSurrendersFocusOnKeystroke(true);
        //Load
    	Enumeration<PropEntry> elements = properties.elements();
    	Enumeration<String> keys = properties.keys();
    	for (int i = 0; i<model.getRowCount();i++) {
    		PropEntry e = (PropEntry)(elements.nextElement());			
    		model.setValueAt(keys.nextElement(), i, 0);
    		model.setValueAt(e.getValue(), i, 1);
    	}
    	
    	propTable.addMouseListener(this);
        
    	tableScrollPane.setViewportView(propTable);
        tableScrollPane.setVisible(true);
        model.isLoaded = true;
	}
	
	private void saveTable(TableModel model,Hashtable<String, PropEntry> properties) {
		//Save table content to selected object
		for (int i = 0; i< model.getRowCount();i++) {
			String key = (String)model.getValueAt(i, 0);
			String value = (String)model.getValueAt(i,1);
			
			PropEntry entr = properties.get(key);
			PropTypes type = entr.getType();
			
			if(FrevoMain.checkType(type, value))
				entr.setValue(value);
		}
	}
	
	private class PropertiesTableModel extends AbstractTableModel {

		private static final long serialVersionUID = 8047616004117865346L;

		private Hashtable<Object,Object> lookup;

		private final int rows;
		private final int columns;
		private final String headers[];
		public boolean isLoaded = false;

		public PropertiesTableModel(String columnHeaders[],int rows) {
		    this.columns = columnHeaders.length;
		    headers = columnHeaders;
		    lookup = new Hashtable<Object,Object>();
		    this.rows = rows;
		}

		public int getColumnCount() {
		    return columns;
		}

		public int getRowCount() {
		    return rows;
		}

		public String getColumnName(int column) {
		    return headers[column];
		}

		public Object getValueAt(int row, int column) {
		    return lookup.get(new Point(row, column));
		}

		public void setValueAt(Object value, int row, int column) {
		    if ((rows < 0) || (columns < 0)) {
		      throw new IllegalArgumentException("Invalid row/column setting");
		    }
		    if ((row < rows) && (column < columns)) {
		      lookup.put(new Point(row, column), value);
		      if (isLoaded) fireTableCellUpdated(row, column);
		    }
		    else throw new Error ("Wrong values");
		}
		  
		boolean[] canEdit = new boolean [] {false, true};

	    public boolean isCellEditable(int rowIndex, int columnIndex) {
	        return canEdit [columnIndex];
	    }
	}

	@Override
	public void windowActivated(WindowEvent e) {
	}


	@Override
	public void windowClosed(WindowEvent e) {
		e.getComponent().setVisible(false);
	}

	@Override
	public void windowClosing(WindowEvent e) {
		//save preferred width
		FrevoMain.componentBrowserParameters[0] = ComponentBrowser.this.getBounds().width;
		FrevoMain.componentBrowserParameters[1] = ComponentBrowser.this.getBounds().height;
		FrevoMain.componentBrowserParameters[2] = topsplitPane.getDividerLocation();
		FrevoMain.componentBrowserParameters[3] = bigsplitPane.getDividerLocation();
		FrevoMain.mainWindow.setEnabled(true);
	}

	@Override
	public void windowDeactivated(WindowEvent e) {}

	@Override
	public void windowDeiconified(WindowEvent e) {}

	@Override
	public void windowIconified(WindowEvent e) {}

	@Override
	public void windowOpened(WindowEvent e) {}

	@Override
	public void mouseClicked(MouseEvent e) {
		int row = propTable.rowAtPoint(e.getPoint());
		int column = propTable.columnAtPoint(e.getPoint());
		if (column == 1) {
			String key = (String)model.getValueAt(row, 0);
			PropEntry entr = properties.get(key);
			PropTypes type = entr.getType();
			if (type == PropTypes.FILE) {
				//open file dialog
				JFileChooser fc = new JFileChooser(FrevoMain.currentDir);
				int returnVal = fc.showDialog(ComponentBrowser.this,"Select file");
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					//change the table instead of the underlying data (??)
					propTable.setValueAt(fc.getSelectedFile().getAbsolutePath(), row, column);
					
					//reload
					model.fireTableCellUpdated(row, column);				
				}
				
			}
		}
	}

	@Override
	public void mouseEntered(MouseEvent e) {}

	@Override
	public void mouseExited(MouseEvent e) {}

	@Override
	public void mousePressed(MouseEvent e) {
		int selRow = tree.getRowForLocation(e.getX(), e.getY());
        
        DefaultMutableTreeNode node = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();
        if (node == null)//Nothing is selected.	
			return;
        
        if(selRow != -1) {
            if(e.getClickCount() == 1) {
            	//load
            	Object nodeInfo = node.getUserObject();
				if (node.isLeaf()) {//it is a component and not an empty category
					if (nodeInfo instanceof ComponentXMLdata ) {
						ComponentXMLdata cdata = (ComponentXMLdata)nodeInfo;
						loadSelected(cdata);
						//select component
						SELECTEDITEM = FrevoMain.getComponentList(componentType).indexOf(cdata);
						confirmButton.setEnabled(true);
					}
				} else {
					//do not change the display
				}
            }
            else if(e.getClickCount() == 2) {
            	if (node.isLeaf()) submitChanges();
            }
        }
	}

	@Override
	public void mouseReleased(MouseEvent e) {}

	@Override
	public void tableChanged(TableModelEvent e) {
		saveTable(model,properties);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == confirmButton) {
			submitChanges();
		} else if (e.getSource() == cancelButton) {
			saveChanges();
			FrevoMain.mainWindow.setEnabled(true);
			ComponentBrowser.this.setVisible(false);
		}
	}

}
