package graphics;

import interfaces.ComponentXMLdata;
import interfaces.FileType;
import interfaces.IMethod;
import interfaces.IProblem;
import interfaces.IRepresentation;
import interfaces.PropEntry;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;
import java.util.Random;
import java.util.Vector;
import java.util.concurrent.ExecutionException;

import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.GroupLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.ToolTipManager;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;

import main.FrevoMain;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

import utils.ComponentCreator;
import utils.NESRandom;

public class FrevoWindow extends JFrame implements Runnable, MouseListener, ActionListener, WindowListener, DocumentListener, KeyListener, ListSelectionListener  {

	private static final long serialVersionUID = -8505581652138492581L;
	
	private final boolean REDIRECTCONSOLE = true;
	
	//Image files
	private BufferedImage iconImg = null;
	ImageIcon nothingIcon = null;
	private ImageIcon problemIcon = null;
	private ImageIcon problem_hIcon = null;
	private ImageIcon methodIcon = null;
	private ImageIcon method_hIcon = null;
	private ImageIcon representationIcon = null;
	private ImageIcon representation_hIcon = null;
	private ImageIcon rankingIcon = null;
	private ImageIcon ranking_hIcon = null;
	private ImageIcon advancedIcon = null;
	private ImageIcon advanced_hIcon = null;
	
	//Panels
	private JPanel configPanel;
	private JPanel advancedPanel;
	private JPanel topPanel;
	private JPanel simcontrolPanel;
	private JPanel statisticPanel;
	private JPanel resultsPanel;
	private JPanel detailsPanel;
	
	//Components
	private JLabel problemButton;
	private JLabel methodButton;
	private JLabel representationButton;
	private JLabel rankingButton;
	private JLabel problemLabel;
	private JLabel methodLabel;
	private JLabel representationLabel;
	private JLabel rankingLabel;
	private JLabel advancedButton;
	private JTextPane advancedLabel;
	private JLabel numberofrunsLabel;
	private IntegerTextField numberofrunsTextField;
	private JLabel seedLabel;
	private IntegerTextField seedTextField;
	private JLabel simnameLabel;
	private JTextField simnameTextField;
	private JButton backButton;
	private JButton replayButton;
	private JButton closeresButton;
	private JTabbedPane tabbedPane;	
	private JProgressBar progressBar;
	private JButton startButton;
	private JButton stopButton;
	private JButton resetButton;
	private JButton consolesaveButton;
	private JButton consoleclearButton;
	private JTable detailsTable;
	
	//Panes
	private JTextPane consoleTextPane;
	private JScrollPane consoleScrollPane;
	private JSplitPane resultsSplitPane;
	private JScrollPane detailsScrollPane;
	
	private JMenuItem aboutItem;
	private JMenuItem loadItem;
	private JMenuItem saveItem;
	private JMenuItem ccreatorItem;
	
	//Component selectors
	ComponentBrowser pBrowser;
	ComponentBrowser mBrowser;
	ComponentBrowser reBrowser;
	ComponentBrowser raBrowser;
	
	private WorkerThread worker; //The worker thread for the method component (swing)
	boolean sessionInProgress = false;
	
	private NESRandom randomholder;
	
	/** Loaded populations */
	private ArrayList<ArrayList<IRepresentation>> populations;
	private ArrayList<JList> jLists;
	
	//streams for console redirection
	private final PipedInputStream pin = new PipedInputStream(); 
	private final PipedInputStream pin2 = new PipedInputStream();
	
	private Thread stdOutReader;
	private Thread stdErrReader;
	StyledDocument consoleDoc;
	Style OutStyle;
	Style ErrStyle;
	
	//Constants
	public final static Dimension iconDim = new Dimension (48,48);
	
	public FrevoWindow() {
		super ("FREVO - Framework for Evolutionary Design "+"v"+FrevoMain.MAJORVERSION+" Rev:"+FrevoMain.MINORVERSION);
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		setBounds(0,0,FrevoMain.mainWindowParameters[0],FrevoMain.mainWindowParameters[1]);// set initial size
		Container con = this.getContentPane(); // inherit main frame
        this.setLocationRelativeTo(null);
        setMinimumSize(new Dimension(500,550));
        
        //set tooltips to appear almost immediately
	    ToolTipManager.sharedInstance().setInitialDelay(10);
        
        //Load images
        try {
        	iconImg = ImageIO.read(new URL("jar:file:./Libraries/resources.jar/!" + "/" + "icon.png"));
        	nothingIcon = new ImageIcon(ClassLoader.getSystemResource("noimage.png"));
        	problemIcon = new ImageIcon(ClassLoader.getSystemResource("problem.png"));
        	problem_hIcon = new ImageIcon(ClassLoader.getSystemResource("problem_h.png"));
        	methodIcon = new ImageIcon(ClassLoader.getSystemResource("method.png"));
        	method_hIcon = new ImageIcon(ClassLoader.getSystemResource("method_h.png"));
        	representationIcon = new ImageIcon(ClassLoader.getSystemResource("representation.png"));
        	representation_hIcon = new ImageIcon(ClassLoader.getSystemResource("representation_h.png"));
        	rankingIcon = new ImageIcon(ClassLoader.getSystemResource("ranking.png"));
        	ranking_hIcon = new ImageIcon(ClassLoader.getSystemResource("ranking_h.png"));
        	advancedIcon = new ImageIcon(ClassLoader.getSystemResource("gear.png"));
        	advanced_hIcon = new ImageIcon(ClassLoader.getSystemResource("gear_h.png"));
        	
        	//set FREVO application icon
            this.setIconImage(iconImg);
        } catch (IOException e) {
       	 	System.err.println("Image loading error!");
       	 	e.printStackTrace();
       	 	System.exit(0);
        }
 
        //**************MENU+ITEMS************************************
	    JMenuBar menuBar = new JMenuBar();
	    
	    JMenu menu = new JMenu("File");
	    menu.setMnemonic(KeyEvent.VK_F);
	    menu.getAccessibleContext().setAccessibleDescription(
        "Access Frevo settings");
	    menuBar.add(menu);
	    
	    //Load menu item
	    loadItem = new JMenuItem ("Open...",KeyEvent.VK_O);
	    loadItem.setAccelerator(KeyStroke.getKeyStroke(
	    		KeyEvent.VK_O, ActionEvent.CTRL_MASK));
	    loadItem.getAccessibleContext().setAccessibleDescription(
	    		"Load FREVO session or results");
	    menu.add(loadItem);
	    loadItem.addActionListener(this);
	    //Save menu item
	    saveItem = new JMenuItem ("Save...",KeyEvent.VK_S);
	    saveItem.setAccelerator(KeyStroke.getKeyStroke(
	    		KeyEvent.VK_S, ActionEvent.CTRL_MASK));
	    saveItem.getAccessibleContext().setAccessibleDescription(
	    		"Save FREVO session");
	    saveItem.addActionListener(this);
	    menu.add(saveItem);
	    
	    //Configuration
	    JMenuItem propertiesItem = new JMenuItem("Properties",KeyEvent.VK_P);
		propertiesItem.setAccelerator(KeyStroke.getKeyStroke(
		KeyEvent.VK_P, ActionEvent.CTRL_MASK));
		propertiesItem.getAccessibleContext().setAccessibleDescription(
		"FREVO Settings and properties");
		propertiesItem.setEnabled(false);
		menu.add(propertiesItem);
		
		//About
		aboutItem = new JMenuItem ("About FREVO");
		aboutItem.setAccelerator(KeyStroke.getKeyStroke(
				KeyEvent.VK_A, ActionEvent.CTRL_MASK));
		aboutItem.addActionListener(this);
				
		//Components menu
		JMenu compmenu = new JMenu("Components");
		compmenu.setMnemonic(KeyEvent.VK_C);
		compmenu.getAccessibleContext().setAccessibleDescription(
        "Access Frevo components");
	    menuBar.add(compmenu);
	    
	    //component creator
	    ccreatorItem = new JMenuItem("Create new component template...",KeyEvent.VK_N);
	    ccreatorItem.setAccelerator(KeyStroke.getKeyStroke(
		KeyEvent.VK_N, ActionEvent.ALT_MASK));
	    ccreatorItem.addActionListener(this);
	    ccreatorItem.getAccessibleContext().setAccessibleDescription(
		"Create new component template");
	    compmenu.add(ccreatorItem);
	    
	    //Help menu
		JMenu helpmenu = new JMenu("Help");
		helpmenu.setMnemonic(KeyEvent.VK_H);
		menuBar.add(helpmenu);
		helpmenu.add(aboutItem);
		
		this.setJMenuBar(menuBar);
		
		//**************PANELS*******************************************
        
        //configuration panel------------------------
        configPanel = new JPanel();
        Dimension leftDim = new Dimension(250,266);
	    configPanel.setMinimumSize(leftDim);
	    configPanel.setMaximumSize(leftDim);
	    	    
	    TitledBorder configtitle = BorderFactory.createTitledBorder(
	    		BorderFactory.createLineBorder(Color.BLACK,2), "Configure Session");
	    configtitle.setTitlePosition(TitledBorder.CENTER);
	    configtitle.setTitleJustification(TitledBorder.CENTER);
	    configPanel.setBorder(configtitle);
	    
	    problemButton = new JLabel(problemIcon);
	    problemButton.setPreferredSize(iconDim);
	    problemButton.setMinimumSize(iconDim);
	    problemButton.setMaximumSize(iconDim);
	    problemButton.setToolTipText("Select and configure Problem component");
	    problemButton.addMouseListener(this);
	    
	    methodButton = new JLabel(methodIcon);
	    methodButton.setPreferredSize(iconDim);
	    methodButton.setMinimumSize(iconDim);
	    methodButton.setMaximumSize(iconDim);
	    methodButton.setEnabled(false);
	    methodButton.addMouseListener(this);
	    
	    representationButton = new JLabel(representationIcon);
	    representationButton.setPreferredSize(iconDim);
	    representationButton.setMinimumSize(iconDim);
	    representationButton.setMaximumSize(iconDim);
	    representationButton.setEnabled(false);
	    representationButton.addMouseListener(this);
	    
	    rankingButton = new JLabel(rankingIcon);
	    rankingButton.setPreferredSize(iconDim);
	    rankingButton.setMinimumSize(iconDim);
	    rankingButton.setMaximumSize(iconDim);
	    rankingButton.setEnabled(false);
	    rankingButton.addMouseListener(this);
	    
	    problemLabel = new JLabel ("Not selected");
	    problemLabel.setMinimumSize(iconDim);
	    methodLabel = new JLabel ("Not selected");
	    methodLabel.setMinimumSize(iconDim);
	    representationLabel = new JLabel ("Not selected");
	    representationLabel.setMinimumSize(iconDim);
	    rankingLabel = new JLabel ("Not selected");
	    rankingLabel.setMinimumSize(iconDim);
	    
	    Dimension prefdim = new Dimension(32,32);
	    advancedButton = new JLabel(advancedIcon);
	    advancedButton.setMinimumSize(prefdim);
	    advancedButton.setPreferredSize(prefdim);
	    advancedButton.setMaximumSize(prefdim);
	    advancedButton.setEnabled(true);
	    advancedButton.addMouseListener(this);
	    
	    advancedLabel = new JTextPane();
	    Dimension advdim = new Dimension(170,32);
	    advancedLabel.setMinimumSize(advdim);
	    advancedLabel.setPreferredSize(advdim);
	    advancedLabel.setMaximumSize(advdim);
	    advancedLabel.setEditable(false);
	    advancedLabel.setContentType("text/html");
	    advancedLabel.setOpaque(false);
	    updateAdvancedLabel();
	    
	    GroupLayout configlayout = new GroupLayout(configPanel);
        configPanel.setLayout(configlayout);
        int gap = 5;
        //horizontal
        configlayout.setHorizontalGroup(configlayout.createSequentialGroup()
        		.addGap(5)
        		.addGroup(configlayout.createParallelGroup(GroupLayout.Alignment.LEADING)
		        		.addGroup(configlayout.createSequentialGroup()
		        				.addComponent(problemButton)
		        				.addGap(gap)
		        				.addComponent(problemLabel))
		        		.addGroup(configlayout.createSequentialGroup()
		        				.addComponent(methodButton)
		        				.addGap(gap)
		        				.addComponent(methodLabel))
		        		.addGroup(configlayout.createSequentialGroup()
		        				.addComponent(representationButton)
		        				.addGap(gap)
		        				.addComponent(representationLabel))
		        		.addGroup(configlayout.createSequentialGroup()
		        				.addComponent(rankingButton)
		        				.addGap(gap)
		        				.addComponent(rankingLabel))
		        		.addGap(5)
		        		.addGroup(configlayout.createSequentialGroup()
		        				.addGap(9)
		        				.addComponent(advancedButton)
		        				.addGap(12)
		        				.addComponent(advancedLabel))
		        		)
		        		.addGap(5)
        );

        //vertical
	    configlayout.setVerticalGroup(configlayout.createParallelGroup()
	    		.addGap(5)
	    		.addGroup(configlayout.createSequentialGroup()
		    		.addGroup(configlayout.createParallelGroup(GroupLayout.Alignment.LEADING)
		    				.addComponent(problemButton)
		    				.addGap(gap)
		    				.addComponent(problemLabel))
		    		.addGroup(configlayout.createParallelGroup(GroupLayout.Alignment.LEADING)
		    				.addComponent(methodButton)
		    				.addGap(gap)
		    				.addComponent(methodLabel))
		    		.addGroup(configlayout.createParallelGroup(GroupLayout.Alignment.LEADING)
		    				.addComponent(representationButton)
		    				.addGap(gap)
		    				.addComponent(representationLabel))
		    		.addGroup(configlayout.createParallelGroup(GroupLayout.Alignment.LEADING)
		    				.addComponent(rankingButton)
		    				.addGap(gap)
		    				.addComponent(rankingLabel))
		    		.addGap(5)
		    		.addGroup(configlayout.createParallelGroup(GroupLayout.Alignment.LEADING)
		    				.addGap(9)
		    				.addComponent(advancedButton)
	        				.addGap(12)
	        				.addComponent(advancedLabel))
	        		)
	        		.addGap(5)
	    );
	    
	    //Panel for additional settings
	    advancedPanel = new JPanel();
	    Dimension advPanelDim = new Dimension(250, 266);
	    advancedPanel.setMinimumSize(advPanelDim);
	    advancedPanel.setMaximumSize(advPanelDim);
	    	    
	    TitledBorder advancedtitle = BorderFactory.createTitledBorder(
	    		BorderFactory.createLineBorder(Color.BLACK,2), "Simulation settings");
	    advancedtitle.setTitlePosition(TitledBorder.CENTER);
	    advancedtitle.setTitleJustification(TitledBorder.CENTER);
	    advancedPanel.setBorder(advancedtitle);
	    
	    JPanel advcontenPanel = new JPanel();
	    
	    numberofrunsLabel = new JLabel("Number of runs:");
	    numberofrunsTextField = new IntegerTextField();
	    numberofrunsTextField.setPreferredSize(new Dimension(100, 24));
	    numberofrunsTextField.setText(Integer.toString(FrevoMain.maxRun));
	    numberofrunsTextField.addKeyListener(this);

		seedLabel = new JLabel("Starting seed:");
		seedTextField = new IntegerTextField();
		seedTextField.setPreferredSize(new Dimension(100, 24));
		seedTextField.setText(Long.toString(FrevoMain.SEED));
		seedTextField.addKeyListener(this);
		
		simnameLabel = new JLabel("Simulation name:");
		simnameTextField = new JTextField();
		simnameTextField.setPreferredSize(new Dimension(100, 24));
		simnameTextField.setText(FrevoMain.customName);
		simnameTextField.getDocument().addDocumentListener(this);
	    
	    GroupLayout advcontLayout = new GroupLayout(advcontenPanel);
	    advcontenPanel.setLayout(advcontLayout);

	    advcontLayout.setHorizontalGroup(advcontLayout.createParallelGroup()
	    		.addGroup(advcontLayout.createSequentialGroup()
	    				.addComponent(numberofrunsLabel)
	    				.addGap(15)
	    				.addComponent(numberofrunsTextField))
	    		.addGap(10)
	    		.addGroup(advcontLayout.createSequentialGroup()
	    				.addComponent(seedLabel)
	    				.addGap(15)
	    				.addComponent(seedTextField))
	    		.addGap(10)
	    		.addGroup(advcontLayout.createSequentialGroup()
	    				.addComponent(simnameLabel)
	    				.addGap(15)
	    				.addComponent(simnameTextField))
	    		.addGap(10)
	    );
	    advcontLayout.setVerticalGroup(advcontLayout.createSequentialGroup()
	    		.addGroup(advcontLayout.createParallelGroup()
	    				.addComponent(numberofrunsLabel)
	    				.addGap(15)
	    				.addComponent(numberofrunsTextField))
	    		.addGap(10)
	    		.addGroup(advcontLayout.createParallelGroup()
	    				.addComponent(seedLabel)
	    				.addGap(15)
	    				.addComponent(seedTextField))
	    		.addGap(10)
	    		.addGroup(advcontLayout.createParallelGroup()
	    				.addComponent(simnameLabel)
	    				.addGap(15)
	    				.addComponent(simnameTextField))
	    		.addGap(10)
	    );
	    
	    backButton = new JButton("Back");
	    backButton.addActionListener(this);
	    
	    JScrollPane advscrollPane = new JScrollPane(advcontenPanel);
	    advscrollPane.setBorder(BorderFactory.createEmptyBorder());
	    advscrollPane.setMinimumSize(new Dimension(240,206));
	    
	    advancedPanel.setLayout(new GridBagLayout());
	    GridBagConstraints c = new GridBagConstraints();
	    
	    c.gridx = 0;
	    c.gridy = 0;
	    c.fill = GridBagConstraints.BOTH;
	    advancedPanel.add(advscrollPane,c);
	    
	    c.gridx = 0;
	    c.gridy = 1;
	    advancedPanel.add(backButton,c);
	    
	    //Panel for statistics and showing results
	    statisticPanel = new JPanel();
	    
	    TitledBorder restitle = BorderFactory.createTitledBorder(
	    		BorderFactory.createLineBorder(Color.BLACK,2), "Statistics");
	    restitle.setTitlePosition(TitledBorder.CENTER);
	    restitle.setTitleJustification(TitledBorder.CENTER);
	    statisticPanel.setBorder(restitle);
	    
	    //Panel for simulation control------------------------------
	    simcontrolPanel = new JPanel();
	    TitledBorder simcontitle = BorderFactory.createTitledBorder(
	    		BorderFactory.createLineBorder(Color.BLACK,2), "Control");
	    simcontitle.setTitlePosition(TitledBorder.CENTER);
	    simcontitle.setTitleJustification(TitledBorder.CENTER);
	    simcontrolPanel.setBorder(simcontitle);
	    
	    simcontrolPanel.setPreferredSize(new Dimension(250, 70));
	    simcontrolPanel.setMinimumSize(new Dimension(250, 70));
	    simcontrolPanel.setMaximumSize(new Dimension(250, 70));

	    startButton = new JButton ();
	    startButton.setToolTipText("Start simulation");
	    startButton.setEnabled(false);
	    startButton.addActionListener(this);
	    
	    stopButton = new JButton ();
	    stopButton.setToolTipText("Stop simulation");
	    stopButton.setEnabled(false);
	    stopButton.addActionListener(this);
	    
	    resetButton = new JButton ();
	    resetButton.setToolTipText("Reset simulation");
	    resetButton.setEnabled(false);
	    resetButton.addActionListener(this);
	    
	    try {
			startButton.setIcon(new ImageIcon(new URL("jar:file:./Libraries/resources.jar/!" + "/" + "play24.png")));
			stopButton.setIcon(new ImageIcon(new URL("jar:file:./Libraries/resources.jar/!" + "/" + "stop24.png")));
			resetButton.setIcon(new ImageIcon(new URL("jar:file:./Libraries/resources.jar/!" + "/" + "reset24.png")));
		} catch (MalformedURLException e1) {
			e1.printStackTrace();
		}
		
		simcontrolPanel.add(startButton);
		simcontrolPanel.add(stopButton);
		simcontrolPanel.add(resetButton);
		
		//panel for showing results
		resultsPanel = new JPanel();
		resultsPanel.setMinimumSize(new Dimension(325,200));
		tabbedPane = new JTabbedPane();
		replayButton = new JButton ("Replay");
		replayButton.addActionListener(this);
		closeresButton = new JButton ("Close");
		closeresButton.addActionListener(this);
		
		JPanel resbtnPanel = new JPanel();//panel for buttons
		resbtnPanel.setMaximumSize(new Dimension(1000,30));
		resbtnPanel.add(replayButton);
		resbtnPanel.add(closeresButton);
		
		GroupLayout resbtnLayout = new GroupLayout(resultsPanel);
		resultsPanel.setLayout(resbtnLayout);

		resbtnLayout.setHorizontalGroup(resbtnLayout.createParallelGroup()
				.addComponent(tabbedPane)
				.addComponent(resbtnPanel)
		);
		resbtnLayout.setVerticalGroup(resbtnLayout.createSequentialGroup()
				.addComponent(tabbedPane)
				.addComponent(resbtnPanel)
		);
	    
	    //panel for console*******************************************
	    JPanel consolePanel = new JPanel();
	    
	    TitledBorder consoletitle = BorderFactory.createTitledBorder(
	    		BorderFactory.createLineBorder(Color.BLACK,2), "Console");
	    consoletitle.setTitlePosition(TitledBorder.CENTER);
	    consoletitle.setTitleJustification(TitledBorder.CENTER);
	    consolePanel.setBorder(consoletitle);
	    
	    consoleTextPane = new JTextPane();
	    consoleTextPane.setEditable(false);
	    
	    consoleScrollPane = new JScrollPane(consoleTextPane);
	    
	    //panel showing details
	    detailsPanel = new JPanel();
	    detailsPanel.setBorder(BorderFactory.createTitledBorder(
	    		BorderFactory.createLineBorder(Color.BLACK,2), "Details"));
	    detailsPanel.setMinimumSize(leftDim);
	    
	    TableModel tm = new HashTableModel(null);
	    detailsTable = new JTable(tm);
	    detailsScrollPane = new JScrollPane(detailsTable);
	    detailsTable.setTableHeader(null);
	    GridLayout detailsLayout = new GridLayout(1,1);
	    detailsPanel.setLayout(detailsLayout);
	    detailsPanel.add(detailsScrollPane);
	    
	    resultsSplitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,resultsPanel,detailsPanel);
	    
	    //styles for different colors
	    consoleDoc = (StyledDocument)consoleTextPane.getDocument();
	    OutStyle = consoleDoc.addStyle("OutStyle", null);
	    StyleConstants.setForeground(OutStyle, Color.BLACK);
	    ErrStyle = consoleDoc.addStyle("ErrStyle", null);
	    StyleConstants.setForeground(ErrStyle, Color.RED);
	    
	    JPanel consolecontrolPanel = new JPanel();
	    Dimension consoleControlDim = new Dimension (30,80);
	    consolecontrolPanel.setMinimumSize(consoleControlDim);
	    consolecontrolPanel.setMaximumSize(consoleControlDim);
	    
	    Dimension buttondim = new Dimension(25,25);
	    
	    consolesaveButton = new JButton();
	    consolesaveButton.setPreferredSize(buttondim);
	    consolesaveButton.setToolTipText("Save console content to file");
	    consolesaveButton.addActionListener(this);
	    
	    consoleclearButton = new JButton();
	    consoleclearButton.setPreferredSize(buttondim);
	    consoleclearButton.setToolTipText("Erase console");
	    consoleclearButton.addActionListener(this);
	    
	    try {
			consolesaveButton.setIcon(new ImageIcon(new URL("jar:file:./Libraries/jlfgr-1_0.jar/!" + "/" + "toolbarButtonGraphics/general/Save24.gif")));
			consoleclearButton.setIcon(new ImageIcon(new URL("jar:file:./Libraries/jlfgr-1_0.jar/!" + "/" + "toolbarButtonGraphics/general/Delete24.gif")));
		} catch (MalformedURLException e1) {
			e1.printStackTrace();
		}
		consolecontrolPanel.add(consolesaveButton);
		consolecontrolPanel.add(consoleclearButton);
		
			    
	    GroupLayout consoleLayout = new GroupLayout(consolePanel);
	    consolePanel.setLayout(consoleLayout);
	    
	    consoleLayout.setHorizontalGroup(consoleLayout.createSequentialGroup()
	    		.addComponent(consoleScrollPane)
	    		.addComponent(consolecontrolPanel)
	    );
	    
	    consoleLayout.setVerticalGroup(consoleLayout.createParallelGroup()
	    		.addComponent(consoleScrollPane)
	    		.addComponent(consolecontrolPanel)
	    );
	    
	    if (REDIRECTCONSOLE) {
			//redirect console
			try {
				PipedOutputStream pout = new PipedOutputStream(this.pin);
				System.setOut(new PrintStream(pout, true));
			} catch (java.io.IOException io) {
				try {
					consoleDoc.insertString(consoleDoc.getLength(),
							"Couldn't redirect STDOUT to console\n"
									+ io.getMessage(), ErrStyle);
				} catch (BadLocationException e) {
					e.printStackTrace();
				}
			} catch (SecurityException se) {
				try {
					consoleDoc.insertString(consoleDoc.getLength(),
							"Couldn't redirect STDOUT to console\n"
									+ se.getMessage(), ErrStyle);
				} catch (BadLocationException e) {
					e.printStackTrace();
				}
			}
			try {
				PipedOutputStream pout2 = new PipedOutputStream(this.pin2);
				System.setErr(new PrintStream(pout2, true));
			} catch (java.io.IOException io) {
				try {
					consoleDoc.insertString(consoleDoc.getLength(),
							"Couldn't redirect STDERR to console\n"
									+ io.getMessage(), ErrStyle);
				} catch (BadLocationException e) {
					e.printStackTrace();
				}
			} catch (SecurityException se) {
				try {
					consoleDoc.insertString(consoleDoc.getLength(),
							"Couldn't redirect STDERR to console\n"
									+ se.getMessage(), ErrStyle);
				} catch (BadLocationException e) {
					e.printStackTrace();
				}
			}
			stdOutReader = new Thread(this);
			stdOutReader.setDaemon(true);
			stdOutReader.start();
			stdErrReader = new Thread(this);
			stdErrReader.setDaemon(true);
			stdErrReader.start();
		}
		//Progressbar
	    progressBar = new JProgressBar();
	    progressBar.setPreferredSize(new Dimension(920, 35));
	    progressBar.setMinimumSize(new Dimension(50, 35));
	    progressBar.setMaximumSize(new Dimension(Integer.MAX_VALUE, 35));
	    progressBar.setValue(0);
	    progressBar.setStringPainted(true);
	    
	    topPanel = new JPanel();
	    GroupLayout topLayout = new GroupLayout(topPanel);
	    topPanel.setLayout(topLayout);
	    
	    topLayout.setHorizontalGroup(topLayout.createSequentialGroup()
	    		.addGroup(topLayout.createParallelGroup()
	    				.addComponent(configPanel)
	    				.addComponent(simcontrolPanel))
	    		.addComponent(statisticPanel)
	    );
	    
	    topLayout.setVerticalGroup(topLayout.createParallelGroup()
	    		.addGroup(topLayout.createSequentialGroup()
	    				.addComponent(configPanel)
	    				.addComponent(simcontrolPanel))
	    		.addComponent(statisticPanel)
	    );
	    
	    JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                topPanel, consolePanel);
	    
	    //************LAYOUT ARRANGEMENT***************************************************
	    //Arrange grouplayout
	    GroupLayout mainlayout = new GroupLayout(con);
        con.setLayout(mainlayout);
        
        mainlayout.setHorizontalGroup(mainlayout.createParallelGroup()
        		.addComponent(splitPane)
        		.addComponent(progressBar)
        );
        
        mainlayout.setVerticalGroup(mainlayout.createSequentialGroup()
        		.addComponent(splitPane)
        		.addComponent(progressBar)      		
        );
        
        this.addWindowListener(this);
        
        System.out.println("Welcome to FREVO "+"v"+FrevoMain.MAJORVERSION+" Rev:"+FrevoMain.MINORVERSION);
	}

	/**
	 * Updates the label of the preferences button
	 */
	void updateAdvancedLabel() {
		String text = "";
		if (FrevoMain.maxRun > 1) text = FrevoMain.maxRun+" runs of <i>"+FrevoMain.customName+"</i><br> with starting seed of "+FrevoMain.SEED;
	    else text = FrevoMain.maxRun+" run of <i>"+FrevoMain.customName+"</i><br> with starting seed of "+FrevoMain.SEED;
	    advancedLabel.setText("<font face=\"ARIAL\" size=\"3\">"+text+"</font>");
	}
	
	public void startSim(IMethod method) {
		//Start worker thread
		worker = new WorkerThread();
		worker.addMethod(method);

		worker.execute();
	}

	public class WorkerThread extends SwingWorker<Void,Integer> {
		
		private IMethod method;
		
		@Override
	    public Void doInBackground() throws Exception {
	        try {
	        	FrevoMain.isRunning = true;
	        	//adjust GUI to simulation mode
	        	startButton.setEnabled(false);
	        	stopButton.setEnabled(true);
	        	resetButton.setEnabled(false);
	        	
	        	problemButton.setEnabled(false);
	        	methodButton.setEnabled(false);
	        	representationButton.setEnabled(false);
	        	rankingButton.setEnabled(false);
	        	
	        	method.run();
				
			} catch (Exception e) {
				e.printStackTrace();
			}
	        return null;
	    }

	    @Override
	    public void done() {
	    	//code to ensure that exceptions are handled and not swallowed
	    	try {
	    		get();
	    	} catch (final InterruptedException ex) {
	    		throw new RuntimeException(ex);
	    	} catch (final ExecutionException ex) {
	    		throw new RuntimeException(ex.getCause());
	    	}
	    	
	    	
	        //When finished with calculations
	    	FrevoMain.isRunning = false;

	    	startButton.setEnabled(true);
	    	stopButton.setEnabled(false);
	    	resetButton.setEnabled(true);
	    	
	    	problemButton.setEnabled(true);
        	methodButton.setEnabled(true);
        	representationButton.setEnabled(true);
        	rankingButton.setEnabled(true);
		    
        	//update saved statistics
        	FrevoMain.updateStats();
        	
	    	if (sessionInProgress) {
	    		System.out.println ("Simulation has been paused");
	    	}
	    	else {
	    		FrevoMain.currentRun++;
	    		if (FrevoMain.maxRun == FrevoMain.currentRun) {
	    			System.out.println ("Simulation finished");
	    		}
	    		else {
	    			//proceed to next run
	    			System.out.println ("Proceeding to the next run");
	    			FrevoMain.runSimulation(FrevoMain.customName,FrevoMain.SEED+(FrevoMain.currentRun));
	    		}
	    		
	    	}
	    	
	    }
	    
	    public void addMethod (IMethod m) {
	    	this.method = m;
	    	method.addSwingWorker(this);
	    }
	    
	    public IMethod getMethod() {
	    	return method;
	    }
	    
	    public void setProgressToPublish (int p) {
	    	publish(p);
	    }
	    
	    protected void process(List<Integer> results) {
	    	Integer res = results.get(results.size() - 1);
	    	if(progressBar != null) progressBar.setValue(res);
	    	
	    	//publish statistics
	    	if (FrevoMain.STATISTICS.size() > 1) {
	    	//TODO fix this 	statisticsPane.drawPanels();
	    	}
	    }    
	}
	
	public synchronized String readLine(PipedInputStream in) throws IOException {
		String input = "";
		do {
			int available=in.available();
			if (available==0) break;
			byte b[]=new byte[available];
			in.read(b);
			input=input+new String(b,0,b.length);														
		} while( !input.endsWith("\n") &&  !input.endsWith("\r\n") /*&& !quit*/);
		return input;
	}
	
	private void scrollPaneToBottom() {

		SwingUtilities.invokeLater(new Runnable() {

		public void run() {
			consoleScrollPane.getVerticalScrollBar().setValue(
			consoleScrollPane.getVerticalScrollBar().getMaximum());
			}
		});
	}

	@Override
	public synchronized void run() {
		try
		{			
			while (Thread.currentThread()==stdOutReader)
			{
				try { this.wait(100);}catch(InterruptedException ie) {}
				if (pin.available()!=0)
				{
					String input=this.readLine(pin);
					consoleDoc.insertString(consoleDoc.getLength(),input,OutStyle);
					scrollPaneToBottom(); 
				}
			}
			
			while (Thread.currentThread()==stdErrReader)
			{
				try { this.wait(100);}catch(InterruptedException ie) {}
				if (pin2.available()!=0)
				{
					String input=this.readLine(pin2);					
					consoleDoc.insertString(consoleDoc.getLength(),input,ErrStyle);
					scrollPaneToBottom();
				}
			}
					
		} catch (Exception e)
		{
			try {
				consoleDoc.insertString(consoleDoc.getLength(),"\nConsole reports an Internal error: "+e,ErrStyle);
			} catch (BadLocationException e1) {
				e1.printStackTrace();
			}		
		}				
	}
	
	//Mouse events

	@Override
	public void mouseClicked(MouseEvent e) {
		if (e.getSource()==problemButton) {
			if (pBrowser == null) {
				pBrowser = new ComponentBrowser(FrevoMain.FREVO_IPROBLEM);
			}
			FrevoWindow.this.setEnabled(false);
			pBrowser.setVisible(true);
		} else if ((e.getSource()==methodButton) && (methodButton.isEnabled())){
			if (mBrowser == null) {
				mBrowser = new ComponentBrowser(FrevoMain.FREVO_IMETHOD);
			}
			FrevoWindow.this.setEnabled(false);
			mBrowser.setVisible(true);
		} else if ((e.getSource()==representationButton) && (representationButton.isEnabled())){
			if (reBrowser == null) {
				reBrowser = new ComponentBrowser(FrevoMain.FREVO_IREPRESENTATION);
			}
			FrevoWindow.this.setEnabled(false);
			reBrowser.setVisible(true);
		} else if ((e.getSource()==rankingButton) && (rankingButton.isEnabled())){
			if (raBrowser == null) {
				raBrowser = new ComponentBrowser(FrevoMain.FREVO_IRANKING);
			}
			FrevoWindow.this.setEnabled(false);
			raBrowser.setVisible(true);
		} else if ((e.getSource()==advancedButton) && (advancedButton.isEnabled())) {
			//change topPanel's layout
			topPanel.removeAll();
			GroupLayout topLayout = new GroupLayout(topPanel);
		    
		    
		    topLayout.setHorizontalGroup(topLayout.createSequentialGroup()
		    		.addGroup(topLayout.createParallelGroup()
		    				.addComponent(advancedPanel)
		    				.addComponent(simcontrolPanel))
		    		.addComponent(statisticPanel)
		    );
		    
		    topLayout.setVerticalGroup(topLayout.createParallelGroup()
		    		.addGroup(topLayout.createSequentialGroup()
		    				.addComponent(advancedPanel)
		    				.addComponent(simcontrolPanel))
		    		.addComponent(statisticPanel)
		    );
		    
		    topPanel.setLayout(topLayout);
		    topPanel.validate();
		} 
		
	}

	@Override
	public void mouseEntered(MouseEvent e) {
		if ((e.getSource()==problemButton) && (problemButton.isEnabled())) {
			JLabel label = (JLabel)(e.getSource());
			label.setIcon(problem_hIcon);
		} else if ((e.getSource()==methodButton) && (methodButton.isEnabled())) {
			JLabel label = (JLabel)(e.getSource());
			label.setIcon(method_hIcon);
		} else if ((e.getSource()==representationButton) && (representationButton.isEnabled())) {
			JLabel label = (JLabel)(e.getSource());
			label.setIcon(representation_hIcon);
		} else if ((e.getSource()==rankingButton) && (rankingButton.isEnabled())) {
			JLabel label = (JLabel)(e.getSource());
			label.setIcon(ranking_hIcon);
		} else if ((e.getSource()==advancedButton) && (advancedButton.isEnabled())) {
			JLabel label = (JLabel)(e.getSource());
			label.setIcon(advanced_hIcon);
		}
		
	}

	@Override
	public void mouseExited(MouseEvent e) {
		if ((e.getSource()==problemButton) && (problemButton.isEnabled())) {
			JLabel label = (JLabel)(e.getSource());
			label.setIcon(problemIcon);
		} else if ((e.getSource()==methodButton) && (methodButton.isEnabled())) {
			JLabel label = (JLabel)(e.getSource());
			label.setIcon(methodIcon);
		} else if ((e.getSource()==representationButton) && (representationButton.isEnabled())) {
			JLabel label = (JLabel)(e.getSource());
			label.setIcon(representationIcon);
		} else if ((e.getSource()==rankingButton) && (rankingButton.isEnabled())) {
			JLabel label = (JLabel)(e.getSource());
			label.setIcon(rankingIcon);
		} else if ((e.getSource()==advancedButton) && (advancedButton.isEnabled())) {
			JLabel label = (JLabel)(e.getSource());
			label.setIcon(advancedIcon);
		}		
	}

	@Override
	public void mousePressed(MouseEvent e) {}

	@Override
	public void mouseReleased(MouseEvent e) {}

	//Actionlistener mostly for mouseclicks
	@Override
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		//about menu item
		if (source == aboutItem) {
			//Display about window
			JOptionPane.showMessageDialog(FrevoWindow.this, "FREVO by Istvan Fehervari and Wilfried Elmenreich\nCopyright (C) 2009\nPlease refer to: www.frevotool.tk", "About FREVO", JOptionPane.PLAIN_MESSAGE);
		} //console save button
		else if (source == consolesaveButton) {
			//save console content
			JFileChooser fc = new JFileChooser(FrevoMain.currentDir);
			fc.setAcceptAllFileFilterUsed(false);
			fc.addChoosableFileFilter(new SaveFilter(new FileType("log","FREVO log file (*.log)")));
			
			File saveFile = null;
			while (true) {
				int returnVal = fc.showDialog(FrevoWindow.this,"Save console");
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					File chosen = fc.getSelectedFile();
					if (!chosen.exists()) {
                          saveFile = chosen;
                          saveConsoleToFile(saveFile);
                          break;
                    }
					else {
						int confirm =
                              JOptionPane.showConfirmDialog(
                            		  FrevoWindow.this,
                                  "Overwrite file? " + chosen.getName());
                          if (confirm == JOptionPane.OK_OPTION) {
                              saveFile = chosen;
                              saveConsoleToFile(saveFile);
                              break;
                          } else if (confirm == JOptionPane.NO_OPTION) {
                              continue;
                          }
                          break;
					}
				}
				else break;
			}
			//console clear button
		} else if (source == consoleclearButton) {
			consoleTextPane.setText("");
		}
		//simulation start button
		else if (source == startButton) {
			if (FrevoMain.isLaunchable && !FrevoMain.isRunning) {
				//new simulation or continue
				if (sessionInProgress) FrevoMain.continueSession(simnameTextField.getText(),randomholder);
				else {
					FrevoMain.currentRun = 0;
					FrevoMain.maxRun = Integer.parseInt(numberofrunsTextField.getText());
					FrevoMain.runSimulation(FrevoMain.customName,FrevoMain.SEED);
				}
			}
			else System.out.println ("ERROR: Session setup is incomplete or simulation is already running!");
		}
		//simulation stop button
		else if (source == stopButton) {
			sessionInProgress = true;
			randomholder = worker.getMethod().getRandom();
			worker.getMethod().pause();
			System.out.println ("Simulation has been stopped by the user, please wait while the method stops");
			stopButton.setEnabled(false);
		}
		//simulation reset button
		else if (source == resetButton) {
			int confirm = JOptionPane.showConfirmDialog(
            		  FrevoWindow.this,
                  "Are you sure you want to reset? Simulation progress will be lost!" );
            if (confirm == JOptionPane.OK_OPTION) {
            	//reset frevo
            	sessionInProgress = false;
            	//TODO statisticsPane.removeAll();
            	FrevoMain.STATISTICS.clear();
            	progressBar.setValue(0);
				System.out.println ("Simulation has been reset");
            }
		}
		//config backbutton (back from extra settings)
		else if (source == backButton) {
			updateAdvancedLabel();
			//return config layout
			topPanel.removeAll();
			advancedButton.setIcon(advancedIcon);
			GroupLayout topLayout = new GroupLayout(topPanel);
		    
		    
		    topLayout.setHorizontalGroup(topLayout.createSequentialGroup()
		    		.addGroup(topLayout.createParallelGroup()
		    				.addComponent(configPanel)
		    				.addComponent(simcontrolPanel))
		    		.addComponent(statisticPanel)
		    );
		    
		    topLayout.setVerticalGroup(topLayout.createParallelGroup()
		    		.addGroup(topLayout.createSequentialGroup()
		    				.addComponent(configPanel)
		    				.addComponent(simcontrolPanel))
		    		.addComponent(statisticPanel)
		    );
		    
		    topPanel.setLayout(topLayout);
		    topPanel.validate();
		}
		//load menu
		else if (source == loadItem) {
			JFileChooser fc = new JFileChooser(FrevoMain.currentDir);
			fc.setAcceptAllFileFilterUsed(false);
			fc.addChoosableFileFilter(new ZREFilter());
			fc.addChoosableFileFilter(new ZSEFilter());
			
			int returnVal = fc.showDialog(FrevoWindow.this,"Open FREVO file");
		    if (returnVal == JFileChooser.APPROVE_OPTION) {
				File loadFile = fc.getSelectedFile();
				
				//Load session part
				Document doc = FrevoMain.loadSession(loadFile);
				
				if (FrevoMain.getExtension(loadFile).equals("zre")) {
			        //Load populations -> Call method's own loader       
					try {
						//Instantiate components
						ComponentXMLdata method = FrevoMain.iMethods.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IMETHOD]);
						ComponentXMLdata representation = FrevoMain.iRepresentations.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IREPRESENTATION]);
						
						IMethod m = (IMethod)method.prepareComponent();
						m.addIRepresentation(representation);
						populations = m.loadfromxml(doc);
						refreshResultList();
					} catch (InstantiationException e1) {
						e1.printStackTrace();
					} catch (IllegalAccessException e1) {
						e1.printStackTrace();
					}
					
					//adjust GUI
					resultsPanel.setBorder(BorderFactory.createTitledBorder(
				    		BorderFactory.createLineBorder(Color.BLACK,2), "Results of "+loadFile.getName()));
					topPanel.removeAll();
					GroupLayout topLayout = new GroupLayout(topPanel);
				    
				    
				    topLayout.setHorizontalGroup(topLayout.createSequentialGroup()
				    		.addComponent(resultsSplitPane)
				    );
				    
				    topLayout.setVerticalGroup(topLayout.createParallelGroup()
				    		.addComponent(resultsSplitPane)
				    );
				    
				    topPanel.setLayout(topLayout);
				    topPanel.validate();
				}
								
				//update GUI
			    simnameTextField.setText(FrevoMain.customName);
				seedTextField.setText(Long.toString(FrevoMain.SEED));
				numberofrunsTextField.setText(Integer.toString(FrevoMain.maxRun));
				
				if (FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IMETHOD] != -1) {
			    	methodLabel.setText(FrevoMain.iMethods.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IMETHOD]).getName());
			    	methodButton.setEnabled(true);
			    }
			    else methodLabel.setText("Not selected");
			    if (FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IRANKING] != -1) {
			    	rankingLabel.setText(FrevoMain.iRankings.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IRANKING]).getName());
			    	rankingButton.setEnabled(true);
			    }
			    else rankingLabel.setText("Not selected");
			    if (FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IREPRESENTATION] != -1) {
			    	representationLabel.setText(FrevoMain.iRepresentations.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IREPRESENTATION]).getName());
			    	representationButton.setEnabled(true);
			    }
			    else representationLabel.setText("Not selected");
			    
			    problemLabel.setText( FrevoMain.iProblems.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IPROBLEM]).getName()  );
			    
			    if (FrevoMain.checkState()) {
			    	FrevoMain.isLaunchable = true;
			    	startButton.setEnabled(true);
			    }
			    else {
			    	FrevoMain.isLaunchable = false;
			    }
		    }			
		}
		//save session
		else if (source == saveItem) {
			if (FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IPROBLEM] != -1) {
				JFileChooser fc = new JFileChooser(FrevoMain.currentDir);
				fc.setAcceptAllFileFilterUsed(false);
				fc.addChoosableFileFilter(new SaveFilter(new FileType("zse","FREVO session files (*.zse)")));
				File saveFile = null;
				while (true) {
					int returnVal = fc.showDialog(FrevoWindow.this,"Save session");
					if (returnVal == JFileChooser.APPROVE_OPTION) {
						File chosen = fc.getSelectedFile();
						if (!chosen.exists()) {
                              saveFile = chosen;
                              saveSessionToFile(saveFile);
                              break;
                        }
						else {
							int confirm =
                                  JOptionPane.showConfirmDialog(
                                		  FrevoWindow.this,
                                      "Overwrite file? " + chosen.getName());
                              if (confirm == JOptionPane.OK_OPTION) {
                                  saveFile = chosen;
                                  saveSessionToFile(saveFile);
                                  break;
                              } else if (confirm == JOptionPane.NO_OPTION) {
                                  continue;
                              }
                              break;
						}
					}
					else break;
				}
			
			}
			else {
				System.err.println ("Cannot save empty session!");
			}
		}
		//replay result
		else if (source == replayButton) {
			//get selected representations
			ArrayList<IRepresentation> candidates = new ArrayList<IRepresentation>();
			for (int l=0;l<jLists.size();l++) {
				JList r = jLists.get(l);
				ListSelectionModel lsm = r.getSelectionModel();
				int minIndex = lsm.getMinSelectionIndex();
		        int maxIndex = lsm.getMaxSelectionIndex();
		        for (int i = minIndex; i <= maxIndex; i++) {
		            if (lsm.isSelectedIndex(i)) {
		            	int selected = r.getSelectedIndex();
		            	IRepresentation rep = populations.get(l).get(selected);
		            	candidates.add(rep);
		            }
		        }
			}
			//throw error message if not good
			ComponentXMLdata problem = FrevoMain.iProblems.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IPROBLEM]);
			Hashtable <String, PropEntry> req = problem.getRequirements();
			PropEntry minc = req.get("minimumCandidates");
			int minimumplayers = Integer.parseInt(minc.getValue());
			PropEntry maxc = req.get("maximumCandidates");
			int maximumplayers = Integer.parseInt(maxc.getValue());
			int selectednumber = candidates.size();
			if ((selectednumber >= minimumplayers) && (selectednumber <= maximumplayers)) {
				try {
					IProblem ip = (IProblem) problem.prepareComponent();
					ip.setRandom(new Random());
					ip.replayWithVisualization((IRepresentation[])candidates.toArray(new IRepresentation[selectednumber]));
				} catch (Exception e1) {
					e1.printStackTrace();
				} 
			} else {
				String help = "";
				if ((minimumplayers == 1) && (maximumplayers == 1)) help = "Please select exactly one candidate!";
				else help = "Please use the SHIFT and CTRL keys to select min. "+minimumplayers+", max. "+maximumplayers+ " representations.";
				JOptionPane.showMessageDialog(this,
					    "The number of selected candidates are invalid. "+help,
					    "Selection error",
					    JOptionPane.ERROR_MESSAGE);
			}			
		}
		//go back
		else if (source == closeresButton) {
			//revert GUI
			topPanel.removeAll();
			GroupLayout topLayout = new GroupLayout(topPanel);
		    
		    
			topLayout.setHorizontalGroup(topLayout.createSequentialGroup()
		    		.addGroup(topLayout.createParallelGroup()
		    				.addComponent(configPanel)
		    				.addComponent(simcontrolPanel))
		    		.addComponent(statisticPanel)
		    );
		    
		    topLayout.setVerticalGroup(topLayout.createParallelGroup()
		    		.addGroup(topLayout.createSequentialGroup()
		    				.addComponent(configPanel)
		    				.addComponent(simcontrolPanel))
		    		.addComponent(statisticPanel)
		    );		    
		    topPanel.setLayout(topLayout);
		    topPanel.validate();
		}
		else if (source == ccreatorItem) {
			new ComponentCreator().setVisible(true);
		}
	}
	
	/** Reloads populations into the result display */
	private void refreshResultList() {
		jLists = new ArrayList<JList>();
		
		tabbedPane.removeAll();
		
		for (int i= 0;i<populations.size();i++) {
			JPanel panel = new JPanel();
			
			String [] elements = new String [populations.get(i).size()];
			for (int u =0;u<elements.length;u++) {
				elements[u] = populations.get(i).get(0).getXMLData().getClassName()+" "+u+" ("+populations.get(i).get(u).getUniqueName()+")";
			}
			
			JList list = new JList(elements);
			ComponentXMLdata problem = FrevoMain.iProblems.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IPROBLEM]);
			Hashtable <String, PropEntry> req = problem.getRequirements();
			PropEntry minc = req.get("minimumCandidates");
			int minimumplayers = Integer.parseInt(minc.getValue());
			if (minimumplayers > 1) list.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
			else list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			list.setLayoutOrientation(JList.VERTICAL);
			list.getSelectionModel().addListSelectionListener(this);
			
			jLists.add(list);
			JScrollPane scrollPane = new JScrollPane(list);
			GridLayout paneLayout = new GridLayout(1,1);
			panel.setLayout(paneLayout);
			panel.add(scrollPane);
						
			tabbedPane.addTab("Population "+i, panel);
		}
		
	}
	
	private void saveSessionToFile(File saveFile) {
		if (!FrevoMain.getExtension(saveFile).equals("zse")) {
			saveFile = new File(saveFile.getAbsolutePath()
					+ ".zse");
		}

		try {
			//Create file to save
			Document doc = DocumentHelper.createDocument();
			doc.addDocType("frevo", null, ".//Components//ISaveSession.dtd");
			Element dfrevo = doc.addElement("frevo");

			//export sessionconfig
			Element sessionconfig =dfrevo.addElement("sessionconfig");
			//custom name
			Element configentry = sessionconfig.addElement("configentry");
			configentry.addAttribute("key", "CustomName");
			configentry.addAttribute("type", "STRING");
			configentry.addAttribute("value", simnameTextField.getText());
			//number of runs
			Element runentry = sessionconfig.addElement("configentry");
			runentry.addAttribute("key", "NumberofRuns");
			runentry.addAttribute("type", "INT");
			runentry.addAttribute("value", numberofrunsTextField.getText());
			//starting seed
			Element seedentry = sessionconfig.addElement("configentry");
			seedentry.addAttribute("key", "StartingSeed");
			seedentry.addAttribute("type", "LONG");
			seedentry.addAttribute("value", seedTextField.getText());

			//export problem
			Element problemsettings = dfrevo.addElement("problem");
			if (FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IPROBLEM] != -1) {
				ComponentXMLdata problem = FrevoMain.iProblems.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IPROBLEM]);
				Vector<String> keys = new Vector<String>(problem.getProperties().keySet());
				problemsettings.addAttribute("class", problem.getClassName());
				for (String k : keys) {
					Element entry = problemsettings.addElement("problementry");
					entry.addAttribute("key", k);
					entry.addAttribute("type", problem.getType(k).toString());
					entry.addAttribute("value", problem.getValue(k));
				}
			}

			//export method
			Element methodsettings = dfrevo.addElement("method");
			if (FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IMETHOD] != -1) {
				ComponentXMLdata method = FrevoMain.iMethods
						.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IMETHOD]);
				Vector<String> keys = new Vector<String>(method
						.getProperties().keySet());
				methodsettings.addAttribute("class", method
						.getClassName());
				for (String k : keys) {
					Element entry = methodsettings
							.addElement("methodentry");
					entry.addAttribute("key", k);
					entry.addAttribute("type", method
							.getType(k).toString());
					entry.addAttribute("value", method
							.getValue(k));
				}
			}

			//export representation
			Element representationsettings = dfrevo
					.addElement("representation");
			if (FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IREPRESENTATION] != -1) {
				ComponentXMLdata representation = FrevoMain.iRepresentations
						.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IREPRESENTATION]);
				Vector<String> keys = new Vector<String>(
						representation.getProperties().keySet());
				representationsettings.addAttribute("class",
						representation.getClassName());
				for (String k : keys) {
					Element entry = representationsettings
							.addElement("representationentry");
					entry.addAttribute("key", k);
					entry.addAttribute("type", representation
							.getType(k).toString());
					entry.addAttribute("value", representation
							.getValue(k));
				}
			}

			//export ranking
			Element rankingsettings = dfrevo
					.addElement("ranking");
			if (FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IRANKING] != -1) {
				ComponentXMLdata ranking = FrevoMain.iRankings
						.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IRANKING]);
				Vector<String> keys = new Vector<String>(
						ranking.getProperties().keySet());
				rankingsettings.addAttribute("class", ranking
						.getClassName());
				for (String k : keys) {
					Element entry = rankingsettings
							.addElement("rankingentry");
					entry.addAttribute("key", k);
					entry.addAttribute("type", ranking.getType(
							k).toString());
					entry.addAttribute("value", ranking
							.getValue(k));
				}
			}

			OutputFormat format = OutputFormat
					.createPrettyPrint();
			format.setLineSeparator(System
					.getProperty("line.separator"));

			FileWriter out = new FileWriter(saveFile);
			BufferedWriter bw = new BufferedWriter(out);
			XMLWriter wr = new XMLWriter(bw, format);
			wr.write(doc);
			wr.close();
			System.out.println("Session saved to "
					+ saveFile.getName());
		} catch (IOException e) {
			System.err.println("Error saving session");
		}
	}

	private void saveConsoleToFile (File saveFile) {
		if (!FrevoMain.getExtension(saveFile).equals("log")) {
			saveFile = new File(saveFile.getAbsolutePath()
					+ ".log");
		}
		try {
			FileWriter out = new FileWriter (saveFile);
			BufferedWriter output = new BufferedWriter(out);
			for (int i=0;i<consoleDoc.getLength();i++) {
				output.write(consoleDoc.getText(i, 1));
			}								
			output.close();
			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (BadLocationException e) {
			e.printStackTrace();
		}		
	}
	
	//WindowListener methods
	@Override
	public void windowOpened(WindowEvent e) {}
	
	@Override
	public void windowIconified(WindowEvent e) {}
	
	@Override
	public void windowDeiconified(WindowEvent e) {}
	
	@Override
	public void windowDeactivated(WindowEvent e) {}
	
	@Override
	public void windowClosing(WindowEvent e) {
		//save preferred width
		FrevoMain.mainWindowParameters[0] = FrevoWindow.this.getBounds().width;
		FrevoMain.mainWindowParameters[1] = FrevoWindow.this.getBounds().height;
		FrevoMain.saveSettings();
		//ask for confirmation
		if (FrevoMain.isRunning) {
			int confirm =
	            JOptionPane.showConfirmDialog(
	          		  FrevoWindow.this,
	                "Are you sure you want to exit FREVO?","Are you sure?",JOptionPane.YES_NO_OPTION);
	        if (confirm == JOptionPane.OK_OPTION) {
	            System.exit(0);        
	        }
		}
		else System.exit(0);
	
	}
	
	@Override
	public void windowClosed(WindowEvent e) {}
	
	@Override
	public void windowActivated(WindowEvent e) {
		//Refresh labels
		if (FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IPROBLEM] != -1) {
			problemLabel.setText(FrevoMain.iProblems.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IPROBLEM]).getName());
			methodButton.setEnabled(true);
		} else {
			problemLabel.setText("Not selected");
		}
		if (FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IMETHOD] != -1) {
			methodLabel.setText(FrevoMain.iMethods.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IMETHOD]).getName());
			representationButton.setEnabled(true);
		} else {
			methodLabel.setText("Not selected");
		}
		if (FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IREPRESENTATION] != -1) {
			representationLabel.setText(FrevoMain.iRepresentations.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IREPRESENTATION]).getName());
			rankingButton.setEnabled(true);
		} else {
			representationLabel.setText("Not selected");
		}
		if (FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IRANKING] != -1) {
			rankingLabel.setText(FrevoMain.iRankings.get(FrevoMain.SELECTED_COMPONENT[FrevoMain.FREVO_IRANKING]).getName());
		} else {
			rankingLabel.setText("Not selected");
		}
		//check if frevo is launchable
		if (FrevoMain.checkState()) {
	    	FrevoMain.isLaunchable = true;
	    	startButton.setEnabled(true);
	    }
	    else {
	    	FrevoMain.isLaunchable = false;
	    }
	}
	
	public static class IntegerTextField extends JTextField {
		private static final long serialVersionUID = -848135456547986252L;
		final static String badchars 
	       = "`~!@#$%^&*()_-+=\\|\"':;?/>.<, ";

	    public void processKeyEvent(KeyEvent ev) {

	        char c = ev.getKeyChar();

	        if((Character.isLetter(c)) || badchars.indexOf(c) > -1) {
	            ev.consume();
	            return;
	        }
	        else super.processKeyEvent(ev);
	    }
	    
	    /** Returns the content as integer */
	    public int getInteger() {
	    	return Integer.parseInt(this.getText());
	    }
	}

	@Override
	public void changedUpdate(DocumentEvent e) {
		if (e.getDocument() == simnameTextField.getDocument()) {
			FrevoMain.customName = simnameTextField.getText();
		}	
	}

	@Override
	public void insertUpdate(DocumentEvent e) {
		if (e.getDocument() == simnameTextField.getDocument()) {
			FrevoMain.customName = simnameTextField.getText();
		}
	}

	@Override
	public void removeUpdate(DocumentEvent e) {
		if (e.getDocument() == simnameTextField.getDocument()) {
			FrevoMain.customName = simnameTextField.getText();
		}
	}

	@Override
	public void keyPressed(KeyEvent e) {}

	@Override
	public void keyReleased(KeyEvent e) {}

	@Override
	public void keyTyped(KeyEvent e) {
		if (e.getSource() == seedTextField) {
			try {
				FrevoMain.SEED = Long.parseLong(seedTextField.getText());
			} catch (NumberFormatException e1) {
				FrevoMain.SEED = 12345;
			}
		} else if (e.getSource() == numberofrunsTextField) {
			if (!(numberofrunsTextField.getText().equals("")))
				FrevoMain.maxRun = Integer.parseInt(numberofrunsTextField.getText());
				else {
					FrevoMain.maxRun = 1;
					numberofrunsTextField.setText(Integer.toString(1));
				}
		}
	}
	
	//list selection event
	@Override
	public void valueChanged(ListSelectionEvent e) {
		ListSelectionModel lsm = (ListSelectionModel)e.getSource();
		int minIndex = lsm.getMinSelectionIndex();
        int maxIndex = lsm.getMaxSelectionIndex();
        for (int i = minIndex; i <= maxIndex; i++) {
            if (lsm.isSelectedIndex(i)) {
            	int selected = jLists.get(tabbedPane.getSelectedIndex()).getSelectedIndex();
            	IRepresentation rep = populations.get(tabbedPane.getSelectedIndex()).get(selected);
            	detailsPanel.setBorder(BorderFactory.createTitledBorder(
        	    		BorderFactory.createLineBorder(Color.BLACK,2), "Details of "+rep.getUniqueName()));
            	HashTableModel tm = (HashTableModel)detailsTable.getModel();
            	tm.setTableModel(rep.getDetails());
            	tm.fireTableDataChanged();
            }
        }	
	}
	
	public static class ZREFilter extends FileFilter {

	    public boolean accept(File f) {
	        if (f.isDirectory()) {
	            return true;
	        }

	        String extension = FrevoMain.getExtension(f);
	        if (extension != null) {
	            if (
	                extension.equals(FrevoMain.FREVO_RESULT_EXTENSION)) { //result file
	                    return true;
	            } else {
	                return false;
	            }
	        }
	        return false;
	    }

	    //The description of this filter
	    public String getDescription() {
	        return "FREVO results files (*.zre)";
	    }
	}
	
	public static class ZSEFilter extends FileFilter {

	    public boolean accept(File f) {
	        if (f.isDirectory()) {
	            return true;
	        }

	        String extension = FrevoMain.getExtension(f);
	        if (extension != null) {
	            if (
	                extension.equals(FrevoMain.FREVO_SESSION_EXTENSION)) { //session file
	                    return true;
	            } else {
	                return false;
	            }
	        }

	        return false;
	    }

	    //The description of this filter
	    public String getDescription() {
	        return "FREVO session files (*.zse)";
	    }
	}
	
	public static class SaveFilter extends FileFilter {
		
		private FileType sfile;
		
		public SaveFilter (FileType savefile) {
			this.sfile = savefile;
		}

	    public boolean accept(File f) {
	        if (f.isDirectory()) {
	            return true;
	        }

	        String extension = FrevoMain.getExtension(f);
	        if (extension != null) {
	            if (
	                extension.equals(sfile.getExtension())) {
	                    return true;
	            } else {
	                return false;
	            }
	        }

	        return false;
	    }
	    
	    public String getExtension() {
	    	return sfile.getExtension();
	    }

	    //The description of this filter
	    public String getDescription() {
	        return sfile.getDescription();
	    }
	}
	
	/**
     * Stores data in a Map by columns. The keys are columns' names,
     * while each value is a array of Objects. This implementation maintains
     * a default ordering based on keys.
     */
    class HashTableModel extends AbstractTableModel {
		private static final long serialVersionUID = 1930986740986471854L;
		Hashtable<String,String> htable;
		
		public HashTableModel(Hashtable<String,String> hash) {
			this.htable = hash;
		}
		
		public void setTableModel(Hashtable<String,String> hash) {
			this.htable = hash;
		}

		@Override
		public int getColumnCount() {
			return 2;
		}

		@Override
		public int getRowCount() {
			if (htable != null) return htable.size();
			else return 0;
		}

		@Override
		public Object getValueAt(int row, int column) {
			if (column == 0) {
                return getKey(row);
			} else {
                return htable.get(getKey(row));
			}
		}
		
		public boolean isCellEditable(int rowIndex, int columnIndex) {
            return false;
        }
		
		private String getKey(int a_index) {
            String retval = "";
            Enumeration<String> e = htable.keys();
            for (int i = 0; i < a_index + 1; i++) {
                    retval = e.nextElement();
            } // for

            return retval;
		}
    }
}
