/*
 * Copyright (C) 2009 Istvan Fehervari, Wilfried Elmenreich
 * Original project page: http://www.frevotool.tk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License Version 3 as published
 * by the Free Software Foundation http://www.gnu.org/licenses/gpl-3.0.txt
 *
 * There is no warranty for this free software. The GPL requires that 
 * modified versions be marked as changed, so that their problems will
 * not be attributed erroneously to authors of previous versions.
 */
package interfaces;

import java.util.ArrayList;
import java.util.Hashtable;

import main.FrevoMain;


public class ComponentXMLdata {
	
	private Class<IComponent> componentClass;
	private String className;
	private String classDir;
	private String description;
	private String imageLocation = null;
	private String name;
	private ArrayList<String> tags = new ArrayList<String>();
	
	private int componentType;
	
	private Hashtable<String, PropEntry> properties = new Hashtable<String, PropEntry>();
	private Hashtable<String, PropEntry> requirements;
	
	public ComponentXMLdata(int ctype) {
		this.componentType = ctype;
	}
	
	public Hashtable<String, PropEntry> getProperties() {
		return properties;
	}
	
	public Hashtable<String, PropEntry> getRequirements() {
		if (componentType == FrevoMain.FREVO_IPROBLEM) {
			
			try {
				IProblem c = (IProblem)componentClass.newInstance();
				c.addProperties(properties);
				this.requirements = c.adjustRequirements(requirements,properties);
				return requirements;
				
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
			
		}
			System.out.println ("ERROR, this is not an IProblem!");
			return null;
		
	}

	public void setProperties(Hashtable<String, PropEntry> properties) {
		this.properties = properties;
	}

	public Class<IComponent> getComponentClass() {
		return componentClass;
	}

	public void setComponentClass(Class<IComponent> componentClass) {
		this.componentClass = componentClass;
	}

	/** Returns the class name of this component */
	public String getClassName() {
		return className;
	}

	public void setClassName(String className) {
		this.className = className;
	}

	public String getClassDir() {
		return classDir;
	}

	public void setClassDir(String classDir) {
		this.classDir = classDir;
	}

	/** Returns the description of this problem */
	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getImageLocation() {
		return imageLocation;
	}

	public void setImageLocation(String imageLocation) {
		this.imageLocation = imageLocation;
	}

	/** Returns a human-readable name of this component */
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	/** Returns a prepared instance of the stored component class*/
	public IComponent prepareComponent() throws InstantiationException, IllegalAccessException {
		IComponent c = componentClass.newInstance();
		c.addProperties(properties);
		c.setXMLData(this);
		return c;
	}
	
	public void setRequirements(Hashtable<String, PropEntry> reqtable) {
		requirements = reqtable;
	}
	
	public PropTypes getType(String key) {
		PropEntry entr = this.properties.get(key);
		if(entr == null)
			throw new IllegalArgumentException("Key "+key+" not found!");
		else
			return entr.getType();
	}
	
	public String getValue(String key) {
		PropEntry entr = this.properties.get(key);
		if(entr == null)
			throw new IllegalArgumentException("Key "+key+" not found!");
		else
			return entr.getValue();
	}
	
	public ArrayList<String> getTags() {
		return tags;
	}
	
	public void setTags(ArrayList<String> tags) {
		this.tags = tags;
	}
	
	public String toString() {
		return getName();
	}
}
