/*
 * Copyright (C) 2009 Istvan Fehervari, Wilfried Elmenreich
 * Original project page: http://www.frevotool.tk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License Version 3 as published
 * by the Free Software Foundation http://www.gnu.org/licenses/gpl-3.0.txt
 *
 * There is no warranty for this free software. The GPL requires that 
 * modified versions be marked as changed, so that their problems will
 * not be attributed erroneously to authors of previous versions.
 */
package interfaces;

import java.util.Hashtable;


/**
 * SuperInterface for FREVO components 
 * @author Istvan Fehervari
 */
public abstract class IComponent {
	
	private Hashtable<String, PropEntry> properties;
	private ComponentXMLdata xmldata;

	/** Returns the class name of this component */
	//public abstract String getClassName();
	
	/** Returns a human-readable name of this component */
	//public abstract String getName();
	
	/** Returns the description of this problem */
	//public abstract String getDescription();
	
	public ComponentXMLdata getXMLData() {
		return this.xmldata;
	}
	
	public void setXMLData (ComponentXMLdata data) {
		this.xmldata = data;
	}
	
	/** Adds the loaded properties to the component */
	@SuppressWarnings("unchecked")
	public void addProperties (Hashtable<String, PropEntry> properties) {
		// TODO: Implement deep copy to also cover string and other objects		
		this.properties = (Hashtable<String, PropEntry>) properties.clone();
	}
	
	/** Return the properties of this component loaded from the component xml file*/
	public Hashtable<String, PropEntry> getProperties() {
		return this.properties;
	}
	
	/**
	 * Returns the type of the given key.
	 * @return the type of the key
	 * @throws IllegalArgumentException if key is not found
	 */
	public PropTypes getType(String key) {
		PropEntry entr = this.properties.get(key);
		if(entr == null)
			throw new IllegalArgumentException("Key "+key+" not found!");
		else
			return entr.getType();
	}
	
	
	/**
	 * Returns the value assigned to this key
	 * @param key
	 * @return
	 */
	public String getValue(String key) {
		PropEntry entr = this.properties.get(key);
		if(entr == null)
			throw new IllegalArgumentException("Key "+key+" not found!");
		else
			return entr.getValue();
	}
	
}
