/*
 * Copyright (C) 2009 Istvan Fehervari, Wilfried Elmenreich
 * Original project page: http://www.frevotool.tk
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License Version 3 as published
 * by the Free Software Foundation http://www.gnu.org/licenses/gpl-3.0.txt
 *
 * There is no warranty for this free software. The GPL requires that 
 * modified versions be marked as changed, so that their problems will
 * not be attributed erroneously to authors of previous versions.
 */
package interfaces;

import graphics.FrevoWindow.WorkerThread;

import java.io.File;
import java.util.ArrayList;

import org.dom4j.Document;
import org.dom4j.Element;

import utils.NESRandom;

public abstract class IMethod extends IComponent implements Runnable{
	
	protected long seed;
	protected NESRandom generator;
	
	protected boolean isRunning = false;
	protected int progress = 0;
	protected WorkerThread sworker;
	protected String outputname;
	protected boolean isInterrupted = false;
	
	protected ComponentXMLdata representation;
	protected ComponentXMLdata problem;
	protected ComponentXMLdata ranking;
	
	/** Sends an interrupt flag to the method. The method should stop executing and return the intermediate results. */
	public void pause() {
		isInterrupted = true;
	}
	
	public boolean isPaused() {
		return isInterrupted;
	}

	public void addIRepresentation (ComponentXMLdata r) {
		this.representation = r;
	}
	
	public void addIRanking (ComponentXMLdata ranking) {
		this.ranking = ranking;
	}

	public void addIProblem (ComponentXMLdata problem) {
		this.problem = problem;		
	}
	
	/** Adds a swingworker to this class */
	public void addSwingWorker(WorkerThread sw) {
		sworker = sw;
	}
	
	/** Returns the workerthread of this class */
	public WorkerThread getWorkerThread() {
		return sworker;
	}
	
	/** Should return the current progress of the method for display purposes. 
	 * @return a float value between 0 and 100 */
	public abstract float getProgress();
	
	public void setRandom (NESRandom rnd) {
		this.generator = rnd;
		//System.out.println ("random adjusted");
	}
	
	/** Sets the initial random seed for the method */
	public void setRandomSeed(long seed) {
		this.seed = seed;
		generator = new NESRandom(seed);
	}
	
	/** Returns the assigned random object of this class */
	public NESRandom getRandom() {
		if (generator == null) {
			NESRandom g = new NESRandom();
			this.seed = g.nextLong();
			generator = new NESRandom(seed);
		}
		return generator;
	}
	
	/** Returns the random seed of this class */
	public long getSeed() {
		return this.seed;
	}
	
	/**  This function is called by the FrevoMain to perform custom saving operation related to this method */
	public abstract void saveResults(Element dfrevo);
	
	/** Sets the progress of the assigned swingworker object to the given value */
	public void setProgress(int p) {
		if (sworker != null) {
			sworker.setProgressToPublish(p);
		}
	}

	/** Adds a string to add to every output filename (function is subject to change) */
	public void addoutputname(String name) {
		this.outputname = name;
	}

	/** Adds a result file to load up */
	public abstract void addResult(File resfile);

	/** Returns the array of representations for visual output from the given xml*/
	public abstract ArrayList<ArrayList<IRepresentation>> loadfromxml(Document doc);
	
	
}
