package utils;

import java.io.ByteArrayInputStream;
import java.io.File;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.io.SAXReader;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

public class SafeSAX {
	
	/** 
	 * Ignorant implementation of a dtd resolver, does not try to read a dtd but delivers
	 * a generic xml dtd description instead
	 */
	static EntityResolver resolver = new EntityResolver() {
	    public InputSource resolveEntity(String publicId, String systemId) {
	        return new InputSource(new ByteArrayInputStream("<?xml version='1.0' encoding='UTF-8'?>".getBytes())); 
	    }
	};
	
	/**
	 * Creates a new SAXReader and tries to load the given file into an XML document
	 * In case the dtd could not be loaded, the process is restarted using a generic 
	 * resolver 
	 * @param loadFile the file to be loaded from
	 * @return the XML document
	 */
	public static Document read(File loadFile) {
		
		Document doc = null;
        boolean error = false;
        
        try {
            SAXReader xmlReader = new SAXReader(false);        
            doc = xmlReader.read(loadFile);
        } catch (OutOfMemoryError mem) {
            System.err.println("Could not import! (Out of memory)");
        } catch (DocumentException e) {
            //System.err.println("Document type description could not be loaded (probably wrong path) - don't worry, using a generic dtd...");
            error = true;
        }
        
        if (error) {
            try {
                SAXReader xmlReader = new SAXReader(false);
                xmlReader.setEntityResolver( resolver );
                doc = xmlReader.read(loadFile);
            } catch (OutOfMemoryError mem) {
                System.err.println("Could not import! (Out of memory)");
            } catch (DocumentException e) {
                String msg = e.getMessage();
                System.err.println("Error while import: DocumentException\n" + msg);
                error = true;
            }
        }
        
        return doc;
	}
}
